@@ -27,7 +27,6 @@
 #include "eab-config.h"
 
 static GObjectClass *ecp_parent_class;
-static GObjectClass *ecph_parent_class;
 
 struct _EABConfigPrivate {
 	guint source_changed_id;
@@ -43,7 +42,7 @@ ecp_init (GObject *o)
 static void
 ecp_target_free (EConfig *ec, EConfigTarget *t)
 {
-	struct _EABConfigPrivate *p = _PRIVATE(ec);
+	struct _EABConfigPrivate *p = _PRIVATE (ec);
 
 	if (ec->target == t) {
 		switch (t->type) {
@@ -51,7 +50,7 @@ ecp_target_free (EConfig *ec, EConfigTarget *t)
 			EABConfigTargetSource *s = (EABConfigTargetSource *)t;
 
 			if (p->source_changed_id) {
-				g_signal_handler_disconnect(s->source, p->source_changed_id);
+				g_signal_handler_disconnect (s->source, p->source_changed_id);
 				p->source_changed_id = 0;
 			}
 			break; }
@@ -71,17 +70,17 @@ ecp_target_free (EConfig *ec, EConfigTarget *t)
 }
 
 static void
-ecp_source_changed(struct _ESource *source, EConfig *ec)
+ecp_source_changed (struct _ESource *source, EConfig *ec)
 {
-	e_config_target_changed(ec, E_CONFIG_TARGET_CHANGED_STATE);
+	e_config_target_changed (ec, E_CONFIG_TARGET_CHANGED_STATE);
 }
 
 static void
 ecp_set_target (EConfig *ec, EConfigTarget *t)
 {
-	struct _EABConfigPrivate *p = _PRIVATE(ec);
+	struct _EABConfigPrivate *p = _PRIVATE (ec);
 
-	((EConfigClass *)ecp_parent_class)->set_target(ec, t);
+	((EConfigClass *)ecp_parent_class)->set_target (ec, t);
 
 	if (t) {
 		switch (t->type) {
@@ -100,7 +99,7 @@ ecp_class_init (GObjectClass *klass)
 	((EConfigClass *)klass)->set_target = ecp_set_target;
 	((EConfigClass *)klass)->target_free = ecp_target_free;
 
-	g_type_class_add_private(klass, sizeof(struct _EABConfigPrivate));
+	g_type_class_add_private (klass, sizeof (struct _EABConfigPrivate));
 }
 
 GType
@@ -128,7 +127,7 @@ eab_config_get_type (void)
 EABConfig *
 eab_config_new (gint type, const gchar *menuid)
 {
-	EABConfig *ecp = g_object_new (eab_config_get_type(), NULL);
+	EABConfig *ecp = g_object_new (eab_config_get_type (), NULL);
 	e_config_construct (&ecp->config, type, menuid);
 	return ecp;
 }
@@ -143,48 +142,3 @@ eab_config_target_new_source (EABConfig *ecp, struct _ESource *source)
 
 	return t;
 }
-
-static const EConfigHookTargetMask ecph_no_masks[] = {
-	{ NULL }
-};
-
-static const EConfigHookTargetMap ecph_targets[] = {
-	{ "source", EAB_CONFIG_TARGET_SOURCE, ecph_no_masks },
-	{ NULL },
-};
-
-static void
-ecph_class_init (EPluginHookClass *klass)
-{
-	gint i;
-
-	klass->id = "org.gnome.evolution.addressbook.config:1.0";
-
-	for (i = 0; ecph_targets[i].type; i++) {
-		e_config_hook_class_add_target_map ((EConfigHookClass *)klass, &ecph_targets[i]);
-	}
-
-	((EConfigHookClass *)klass)->config_class = g_type_class_ref (eab_config_get_type ());
-}
-
-GType
-eab_config_hook_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (EABConfigHookClass),
-			NULL, NULL,
-			(GClassInitFunc) ecph_class_init,
-			NULL, NULL,
-			sizeof (EABConfigHook), 0,
-			(GInstanceInitFunc) NULL
-		};
-
-		ecph_parent_class = g_type_class_ref (e_config_hook_get_type ());
-		type = g_type_register_static (e_config_hook_get_type (), "EABConfigHook", &info, 0);
-	}
-
-	return type;
-}
