@@ -10,7 +10,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>  
+ * License along with the program; if not, see <http://www.gnu.org/licenses/>
  *
  *
  * Authors:
@@ -32,7 +32,8 @@
 typedef enum {
 	ALARM_NOTIFY_CLOSE,
 	ALARM_NOTIFY_SNOOZE,
-	ALARM_NOTIFY_EDIT
+	ALARM_NOTIFY_EDIT,
+	ALARM_NOTIFY_DISMISS
 } AlarmNotifyResult;
 
 typedef struct _AlarmNotificationsDialog AlarmNotificationsDialog;
@@ -42,7 +43,7 @@ struct _AlarmNotificationsDialog
 	GtkWidget *treeview;
 };
 
-typedef void (* AlarmNotifyFunc) (AlarmNotifyResult result, int snooze_mins, gpointer data);
+typedef void (* AlarmNotifyFunc) (AlarmNotifyResult result, gint snooze_mins, gpointer data);
 
 AlarmNotificationsDialog *
 notified_alarms_dialog_new (void);
@@ -50,8 +51,8 @@ notified_alarms_dialog_new (void);
 GtkTreeIter
 add_alarm_to_notified_alarms_dialog (AlarmNotificationsDialog *na, time_t trigger,
 				time_t occur_start, time_t occur_end,
-				ECalComponentVType vtype, const char *summary,
-				const char *description, const char *location,
+				ECalComponentVType vtype, const gchar *summary,
+				const gchar *description, const gchar *location,
 				AlarmNotifyFunc func, gpointer func_data);
 
 
