@@ -27,11 +27,15 @@
 
 #include <gio/gio.h>
 #include <glib/gi18n.h>
+#include <libsoup/soup.h>
 #include <libecal/e-cal-component.h>
 #include <libecal/e-cal-util.h>
 #include <libecal/e-cal-time-util.h>
 #include <libedataserver/e-data-server-util.h>
+#include <libedataserver/e-proxy.h>
+#include <libedataserverui/e-passwords.h>
 #include <e-util/e-extensible.h>
+#include <e-util/e-util-enumtypes.h>
 #include "itip-utils.h"
 #include "e-meeting-utils.h"
 #include "e-meeting-attendee.h"
@@ -51,6 +55,11 @@ struct _EMeetingStorePrivate {
 	ECal *client;
 	icaltimezone *zone;
 
+	gint default_reminder_interval;
+	EDurationType default_reminder_units;
+
+	gint week_start_day;
+
 	gchar *fb_uri;
 
 	GPtrArray *refresh_queue;
@@ -84,8 +93,11 @@ struct _EMeetingStoreQueueData {
 enum {
 	PROP_0,
 	PROP_CLIENT,
+	PROP_DEFAULT_REMINDER_INTERVAL,
+	PROP_DEFAULT_REMINDER_UNITS,
 	PROP_FREE_BUSY_TEMPLATE,
-	PROP_TIMEZONE
+	PROP_TIMEZONE,
+	PROP_WEEK_START_DAY
 };
 
 /* Forward Declarations */
@@ -539,6 +551,25 @@ e_meeting_store_set_value (EMeetingStore *store, gint row, gint col, const gchar
 	}
 }
 
+struct FindAttendeeData
+{
+	EMeetingAttendee *find;
+	EMeetingStoreQueueData *qdata;
+};
+
+static void
+find_attendee_cb (gpointer key, gpointer value, gpointer user_data)
+{
+	EMeetingStoreQueueData *qdata = value;
+	struct FindAttendeeData *fad = user_data;
+
+	g_return_if_fail (qdata != NULL);
+	g_return_if_fail (fad != NULL);
+
+	if (qdata->attendee == fad->find)
+		fad->qdata = qdata;
+}
+
 static void
 refresh_queue_remove (EMeetingStore *store, EMeetingAttendee *attendee)
 {
@@ -551,6 +582,17 @@ refresh_queue_remove (EMeetingStore *store, EMeetingAttendee *attendee)
 	qdata = g_hash_table_lookup (
 		priv->refresh_data, itip_strip_mailto (
 		e_meeting_attendee_get_address (attendee)));
+	if (!qdata) {
+		struct FindAttendeeData fad = { 0 };
+
+		fad.find = attendee;
+		fad.qdata = NULL;
+
+		g_hash_table_foreach (priv->refresh_data, find_attendee_cb, &fad);
+
+		qdata = fad.qdata;
+	}
+
 	if (qdata) {
 		g_mutex_lock (priv->mutex);
 		g_hash_table_remove (
@@ -580,6 +622,18 @@ meeting_store_set_property (GObject *object,
 				g_value_get_object (value));
 			return;
 
+		case PROP_DEFAULT_REMINDER_INTERVAL:
+			e_meeting_store_set_default_reminder_interval (
+				E_MEETING_STORE (object),
+				g_value_get_int (value));
+			return;
+
+		case PROP_DEFAULT_REMINDER_UNITS:
+			e_meeting_store_set_default_reminder_units (
+				E_MEETING_STORE (object),
+				g_value_get_enum (value));
+			return;
+
 		case PROP_FREE_BUSY_TEMPLATE:
 			e_meeting_store_set_free_busy_template (
 				E_MEETING_STORE (object),
@@ -591,6 +645,12 @@ meeting_store_set_property (GObject *object,
 				E_MEETING_STORE (object),
 				g_value_get_pointer (value));
 			return;
+
+		case PROP_WEEK_START_DAY:
+			e_meeting_store_set_week_start_day (
+				E_MEETING_STORE (object),
+				g_value_get_int (value));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -610,6 +670,20 @@ meeting_store_get_property (GObject *object,
 				E_MEETING_STORE (object)));
 			return;
 
+		case PROP_DEFAULT_REMINDER_INTERVAL:
+			g_value_set_int (
+				value,
+				e_meeting_store_get_default_reminder_interval (
+				E_MEETING_STORE (object)));
+			return;
+
+		case PROP_DEFAULT_REMINDER_UNITS:
+			g_value_set_enum (
+				value,
+				e_meeting_store_get_default_reminder_units (
+				E_MEETING_STORE (object)));
+			return;
+
 		case PROP_FREE_BUSY_TEMPLATE:
 			g_value_set_string (
 				value,
@@ -623,6 +697,13 @@ meeting_store_get_property (GObject *object,
 				e_meeting_store_get_timezone (
 				E_MEETING_STORE (object)));
 			return;
+
+		case PROP_WEEK_START_DAY:
+			g_value_set_int (
+				value,
+				e_meeting_store_get_week_start_day (
+				E_MEETING_STORE (object)));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -683,6 +764,29 @@ e_meeting_store_class_init (EMeetingStoreClass *class)
 			E_TYPE_CAL,
 			G_PARAM_READWRITE));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_DEFAULT_REMINDER_INTERVAL,
+		g_param_spec_int (
+			"default-reminder-interval",
+			"Default Reminder Interval",
+			NULL,
+			G_MININT,
+			G_MAXINT,
+			0,
+			G_PARAM_READWRITE));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_DEFAULT_REMINDER_UNITS,
+		g_param_spec_enum (
+			"default-reminder-units",
+			"Default Reminder Units",
+			NULL,
+			E_TYPE_DURATION_TYPE,
+			E_DURATION_MINUTES,
+			G_PARAM_READWRITE));
+
 	g_object_class_install_property (
 		object_class,
 		PROP_FREE_BUSY_TEMPLATE,
@@ -701,6 +805,18 @@ e_meeting_store_class_init (EMeetingStoreClass *class)
 			"Timezone",
 			NULL,
 			G_PARAM_READWRITE));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_WEEK_START_DAY,
+		g_param_spec_int (
+			"week-start-day",
+			"Week Start Day",
+			NULL,
+			0,  /* Monday */
+			6,  /* Sunday */
+			0,
+			G_PARAM_READWRITE));
 }
 
 static void
@@ -710,7 +826,8 @@ e_meeting_store_init (EMeetingStore *store)
 
 	store->priv->attendees = g_ptr_array_new ();
 	store->priv->refresh_queue = g_ptr_array_new ();
-	store->priv->refresh_data = g_hash_table_new (g_str_hash, g_str_equal);
+	store->priv->refresh_data = g_hash_table_new_full (
+		g_str_hash, g_str_equal, g_free, NULL);
 
 	store->priv->mutex = g_mutex_new ();
 
@@ -752,6 +869,44 @@ e_meeting_store_set_client (EMeetingStore *store,
 	g_object_notify (G_OBJECT (store), "client");
 }
 
+gint
+e_meeting_store_get_default_reminder_interval (EMeetingStore *store)
+{
+	g_return_val_if_fail (E_IS_MEETING_STORE (store), 0);
+
+	return store->priv->default_reminder_interval;
+}
+
+void
+e_meeting_store_set_default_reminder_interval (EMeetingStore *store,
+                                               gint default_reminder_interval)
+{
+	g_return_if_fail (E_IS_MEETING_STORE (store));
+
+	store->priv->default_reminder_interval = default_reminder_interval;
+
+	g_object_notify (G_OBJECT (store), "default-reminder-interval");
+}
+
+EDurationType
+e_meeting_store_get_default_reminder_units (EMeetingStore *store)
+{
+	g_return_val_if_fail (E_IS_MEETING_STORE (store), 0);
+
+	return store->priv->default_reminder_units;
+}
+
+void
+e_meeting_store_set_default_reminder_units (EMeetingStore *store,
+                                            EDurationType default_reminder_units)
+{
+	g_return_if_fail (E_IS_MEETING_STORE (store));
+
+	store->priv->default_reminder_units = default_reminder_units;
+
+	g_object_notify (G_OBJECT (store), "default-reminder-units");
+}
+
 const gchar *
 e_meeting_store_get_free_busy_template (EMeetingStore *store)
 {
@@ -791,6 +946,25 @@ e_meeting_store_set_timezone (EMeetingStore *store,
 	g_object_notify (G_OBJECT (store), "timezone");
 }
 
+gint
+e_meeting_store_get_week_start_day (EMeetingStore *store)
+{
+	g_return_val_if_fail (E_IS_MEETING_STORE (store), 0);
+
+	return store->priv->week_start_day;
+}
+
+void
+e_meeting_store_set_week_start_day (EMeetingStore *store,
+                                    gint week_start_day)
+{
+	g_return_if_fail (E_IS_MEETING_STORE (store));
+
+	store->priv->week_start_day = week_start_day;
+
+	g_object_notify (G_OBJECT (store), "week-start-day");
+}
+
 static void
 attendee_changed_cb (EMeetingAttendee *attendee, gpointer data)
 {
@@ -1475,8 +1649,8 @@ refresh_queue_add (EMeetingStore *store, gint row,
 		g_ptr_array_add (qdata->data, data);
 
 		g_hash_table_insert (
-			priv->refresh_data, (gpointer) itip_strip_mailto (
-			e_meeting_attendee_get_address (attendee)), qdata);
+			priv->refresh_data, g_strdup (itip_strip_mailto (
+			e_meeting_attendee_get_address (attendee))), qdata);
 	} else {
 		if (e_meeting_time_compare_times (start, &qdata->start) == -1)
 			qdata->start = *start;
@@ -1534,6 +1708,188 @@ async_read (GObject *source_object, GAsyncResult *res, gpointer data)
 	}
 }
 
+static void
+soup_authenticate (SoupSession *session,
+                   SoupMessage *msg,
+                   SoupAuth *auth,
+                   gboolean retrying,
+                   gpointer data)
+{
+	SoupURI *suri;
+	const gchar *orig_uri;
+	gboolean tried = FALSE;
+
+	g_return_if_fail (msg != NULL);
+	g_return_if_fail (auth != NULL);
+
+	orig_uri = g_object_get_data (G_OBJECT (msg), "orig-uri");
+	g_return_if_fail (orig_uri != NULL);
+
+	suri = soup_uri_new (orig_uri);
+	if (!suri)
+		return;
+
+	if (!suri->user || !*suri->user) {
+		soup_uri_free (suri);
+		return;
+	}
+
+	if (!retrying) {
+		if (suri->password) {
+			soup_auth_authenticate (auth, suri->user, suri->password);
+			tried = TRUE;
+		} else {
+			gchar *password;
+
+			password = e_passwords_get_password ("Calendar", orig_uri);
+			if (password) {
+				soup_auth_authenticate (auth, suri->user, password);
+				tried = TRUE;
+
+				memset (password, 0, strlen (password));
+				g_free (password);
+			}
+		}
+	}
+
+	if (!tried) {
+		gboolean remember = FALSE;
+		gchar *password, *bold_host, *bold_user;
+		GString *description;
+
+		bold_host = g_strconcat ("<b>", suri->host, "</b>", NULL);
+		bold_user = g_strconcat ("<b>", suri->user, "</b>", NULL);
+
+		description = g_string_new ("");
+
+		g_string_append_printf (
+			description, _("Enter password to access "
+			"free/busy information on server %s as user %s"),
+			bold_host, bold_user);
+
+		g_free (bold_host);
+		g_free (bold_user);
+
+		if (retrying && msg->reason_phrase && *msg->reason_phrase) {
+			g_string_append (description, "\n");
+			g_string_append_printf (description, _("Failure reason: %s"), msg->reason_phrase);
+		}
+
+		password = e_passwords_ask_password (
+			_("Enter password"), "Calendar", orig_uri,
+			description->str, E_PASSWORDS_REMEMBER_FOREVER |
+			E_PASSWORDS_SECRET | E_PASSWORDS_ONLINE |
+			(retrying ? E_PASSWORDS_REPROMPT : 0),
+			&remember, NULL);
+
+		g_string_free (description, TRUE);
+
+		if (password) {
+			soup_auth_authenticate (auth, suri->user, password);
+			tried = TRUE;
+
+			memset (password, 0, strlen (password));
+			g_free (password);
+		}
+	}
+
+	soup_uri_free (suri);
+}
+
+static void
+redirect_handler (SoupMessage *msg, gpointer user_data)
+{
+	if (SOUP_STATUS_IS_REDIRECTION (msg->status_code)) {
+		SoupSession *soup_session = user_data;
+		SoupURI *new_uri;
+		const gchar *new_loc;
+
+		new_loc = soup_message_headers_get (msg->response_headers, "Location");
+		if (!new_loc)
+			return;
+
+		new_uri = soup_uri_new_with_base (soup_message_get_uri (msg), new_loc);
+		if (!new_uri) {
+			soup_message_set_status_full (msg,
+						      SOUP_STATUS_MALFORMED,
+						      "Invalid Redirect URL");
+			return;
+		}
+
+		soup_message_set_uri (msg, new_uri);
+		soup_session_requeue_message (soup_session, msg);
+
+		soup_uri_free (new_uri);
+	}
+}
+
+static void
+soup_msg_ready_cb (SoupSession *session, SoupMessage *msg, gpointer user_data)
+{
+	EMeetingStoreQueueData *qdata = user_data;
+
+	g_return_if_fail (session != NULL);
+	g_return_if_fail (msg != NULL);
+	g_return_if_fail (qdata != NULL);
+
+	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		qdata->string = g_string_new_len (
+			msg->response_body->data,
+			msg->response_body->length);
+		process_free_busy (qdata, qdata->string->str);
+	} else {
+		g_warning (
+			"Unable to access free/busy url: %s",
+			msg->reason_phrase && *msg->reason_phrase ?
+			msg->reason_phrase : (soup_status_get_phrase (
+			msg->status_code) ? soup_status_get_phrase (
+			msg->status_code) : "Unknown error"));
+		process_callbacks (qdata);
+	}
+}
+
+static void
+download_with_libsoup (const gchar *uri, EMeetingStoreQueueData *qdata)
+{
+	SoupSession *session;
+	SoupMessage *msg;
+	EProxy *proxy;
+
+	g_return_if_fail (uri != NULL);
+	g_return_if_fail (qdata != NULL);
+
+	msg = soup_message_new (SOUP_METHOD_GET, uri);
+	if (!msg) {
+		g_warning ("Unable to access free/busy url '%s'; malformed?", uri);
+		process_callbacks (qdata);
+		return;
+	}
+
+	g_object_set_data_full (G_OBJECT (msg), "orig-uri", g_strdup (uri), g_free);
+
+	session = soup_session_async_new ();
+	g_signal_connect (session, "authenticate", G_CALLBACK (soup_authenticate), NULL);
+
+	proxy = e_proxy_new ();
+	e_proxy_setup_proxy (proxy);
+
+	if (e_proxy_require_proxy_for_uri (proxy, uri)) {
+		SoupURI *proxy_uri;
+
+		proxy_uri = e_proxy_peek_uri_for (proxy, uri);
+		g_object_set (session, SOUP_SESSION_PROXY_URI, proxy_uri, NULL);
+	}
+
+	g_object_unref (proxy);
+
+	soup_message_set_flags (msg, SOUP_MESSAGE_NO_REDIRECT);
+	soup_message_add_header_handler (
+		msg, "got_body", "Location",
+		G_CALLBACK (redirect_handler), session);
+	soup_message_headers_append (msg->request_headers, "Connection", "close");
+	soup_session_queue_message (session, msg, soup_msg_ready_cb, qdata);
+}
+
 static void
 start_async_read (const gchar *uri, gpointer data)
 {
@@ -1552,16 +1908,25 @@ start_async_read (const gchar *uri, gpointer data)
 
 	istream = G_INPUT_STREAM (g_file_read (file, NULL, &error));
 
+	if (error && g_error_matches (error, SOUP_HTTP_ERROR, SOUP_STATUS_UNAUTHORIZED)) {
+		download_with_libsoup (uri, qdata);
+		g_object_unref (file);
+		g_error_free (error);
+		return;
+	}
+
 	if (error) {
 		g_warning ("Unable to access free/busy url: %s", error->message);
 		g_error_free (error);
 		process_callbacks (qdata);
+		g_object_unref (file);
 		return;
 	}
 
-	if (!istream)
+	if (!istream) {
 		process_callbacks (qdata);
-	else
+		g_object_unref (file);
+	} else
 		g_input_stream_read_async (
 			istream, qdata->buffer, BUF_SIZE - 1,
 			G_PRIORITY_DEFAULT, NULL, async_read, qdata);
