@@ -70,20 +70,20 @@ rule_editor_add_undo (ERuleEditor *editor,
                       gint rank,
                       gint newrank)
 {
-        ERuleEditorUndo *undo;
+	ERuleEditorUndo *undo;
 
-        if (!editor->undo_active && enable_undo) {
-                undo = g_malloc0 (sizeof (*undo));
-                undo->rule = rule;
-                undo->type = type;
-                undo->rank = rank;
-                undo->newrank = newrank;
+	if (!editor->undo_active && enable_undo) {
+		undo = g_malloc0 (sizeof (*undo));
+		undo->rule = rule;
+		undo->type = type;
+		undo->rank = rank;
+		undo->newrank = newrank;
 
-                undo->next = editor->undo_log;
-                editor->undo_log = undo;
-        } else {
-                g_object_unref (rule);
-        }
+		undo->next = editor->undo_log;
+		editor->undo_log = undo;
+	} else {
+		g_object_unref (rule);
+	}
 }
 
 static void
@@ -263,9 +263,6 @@ rule_add (GtkWidget *widget, ERuleEditor *editor)
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_OK, GTK_RESPONSE_OK,
 				NULL);
-#if !GTK_CHECK_VERSION(2,90,7)
-	g_object_set (editor->dialog, "has-separator", FALSE, NULL);
-#endif
 
 	gtk_window_set_title ((GtkWindow *) editor->dialog, _("Add Rule"));
 	gtk_window_set_default_size (GTK_WINDOW (editor->dialog), 650, 400);
@@ -352,16 +349,11 @@ rule_edit (GtkWidget *widget, ERuleEditor *editor)
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_OK, GTK_RESPONSE_OK,
 				NULL);
-#if !GTK_CHECK_VERSION(2,90,7)
-	g_object_set (editor->dialog, "has-separator", FALSE, NULL);
-#endif
 
 	gtk_window_set_title ((GtkWindow *) editor->dialog, _("Edit Rule"));
 	gtk_window_set_default_size (GTK_WINDOW (editor->dialog), 650, 400);
 	gtk_window_set_resizable (GTK_WINDOW (editor->dialog), TRUE);
-	gtk_widget_set_parent_window (
-		GTK_WIDGET (editor->dialog),
-		gtk_widget_get_window (GTK_WIDGET (editor)));
+	gtk_window_set_transient_for (GTK_WINDOW (editor->dialog), GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (editor))));
 	gtk_container_set_border_width ((GtkContainer *) editor->dialog, 6);
 
 	content_area = gtk_dialog_get_content_area (GTK_DIALOG (editor->dialog));
@@ -569,17 +561,18 @@ rule_editor_finalize (GObject *object)
 }
 
 static void
-rule_editor_destroy (GtkObject *gtk_object)
+rule_editor_dispose (GObject *object)
 {
-	ERuleEditor *editor = E_RULE_EDITOR (gtk_object);
+	ERuleEditor *editor = E_RULE_EDITOR (object);
 
 	if (editor->dialog != NULL) {
 		gtk_widget_destroy (GTK_WIDGET (editor->dialog));
 		editor->dialog = NULL;
 	}
 
-	/* Chain up to parent's destroy() method. */
-	GTK_OBJECT_CLASS (e_rule_editor_parent_class)->destroy (gtk_object);
+	/* Chain up to parent's dispose() method. */
+	if (G_OBJECT_CLASS (e_rule_editor_parent_class)->dispose)
+		G_OBJECT_CLASS (e_rule_editor_parent_class)->dispose (object);
 }
 
 static void
@@ -644,15 +637,12 @@ static void
 e_rule_editor_class_init (ERuleEditorClass *class)
 {
 	GObjectClass *object_class;
-	GtkObjectClass *gtk_object_class;
 
 	g_type_class_add_private (class, sizeof (ERuleEditorPrivate));
 
 	object_class = G_OBJECT_CLASS (class);
 	object_class->finalize = rule_editor_finalize;
-
-	gtk_object_class = GTK_OBJECT_CLASS (class);
-	gtk_object_class->destroy = rule_editor_destroy;
+	object_class->dispose = rule_editor_dispose;
 
 	class->set_source = rule_editor_set_source;
 	class->set_sensitive = rule_editor_set_sensitive;
@@ -852,9 +842,6 @@ e_rule_editor_construct (ERuleEditor *editor,
 		G_CALLBACK (editor_response), editor);
 	rule_editor_set_source (editor, source);
 
-#if !GTK_CHECK_VERSION(2,90,7)
-	g_object_set (editor, "has-separator", FALSE, NULL);
-#endif
 	gtk_dialog_add_buttons ((GtkDialog *) editor,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_OK, GTK_RESPONSE_OK,
