@@ -402,6 +402,9 @@ composer_presend_check_plugins (EMsgComposer *composer)
 	/* A non-NULL value for this key means the check failed. */
 	data = g_object_get_data (G_OBJECT (composer), "presend_check_status");
 
+	/* Clear the value in case we have to run these checks again. */
+	g_object_set_data (G_OBJECT (composer), "presend_check_status", NULL);
+
 	return (data == NULL);
 }
 
@@ -489,7 +492,21 @@ composer_send_completed (EMailSession *session,
 		goto exit;
 	}
 
-	if (error != NULL) {
+	/* Post-processing errors are shown in the shell window. */
+	if (g_error_matches (error, E_MAIL_ERROR, E_MAIL_ERROR_POST_PROCESSING)) {
+		EAlert *alert;
+		EShell *shell;
+
+		shell = e_msg_composer_get_shell (context->composer);
+
+		alert = e_alert_new (
+			"mail-composer:send-post-processing-error",
+			error->message, NULL);
+		e_shell_submit_alert (shell, alert);
+		g_object_unref (alert);
+
+	/* All other errors are shown in the composer window. */
+	} else if (error != NULL) {
 		gint response;
 
 		/* Clear the activity bar before
@@ -1126,8 +1143,8 @@ edit_message (EShell *shell,
 typedef enum { QUOTING_ATTRIBUTION, QUOTING_FORWARD, QUOTING_ORIGINAL } QuotingTextEnum;
 
 static struct {
-	const gchar* gconf_key;
-	const gchar* message;
+	const gchar * gconf_key;
+	const gchar * message;
 } conf_messages[] = {
 	[QUOTING_ATTRIBUTION] =
 		{ "/apps/evolution/mail/composer/message_attribution",
