@@ -35,8 +35,6 @@
 
 #include <glib/gi18n.h>
 
-#include "mail-config.h"
-#include "mail-session.h"
 #include "mail-tools.h"
 #include "mail-mt.h"
 #include "mail-ops.h"
@@ -65,6 +63,7 @@ struct _EMFolderTreeModelPrivate {
 	GtkTreeSelection *selection;  /* weak reference */
 
 	EAccountList *accounts;
+	EMailSession *session;
 
 	/* EAccount -> EMFolderTreeStoreInfo */
 	GHashTable *account_index;
@@ -82,7 +81,8 @@ struct _EMFolderTreeModelPrivate {
 
 enum {
 	PROP_0,
-	PROP_SELECTION
+	PROP_SELECTION,
+	PROP_SESSION
 };
 
 enum {
@@ -97,6 +97,8 @@ extern CamelStore *vfolder_store;
 static gpointer parent_class;
 static guint signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE (EMFolderTreeModel, em_folder_tree_model, GTK_TYPE_TREE_STORE)
+
 static void
 store_info_free (EMFolderTreeModelStoreInfo *si)
 {
@@ -204,10 +206,13 @@ account_changed_cb (EAccountList *accounts,
                     EMFolderTreeModel *model)
 {
 	EMFolderTreeModelStoreInfo *si;
+	EMailSession *session;
 	CamelProvider *provider;
 	CamelStore *store;
 	gchar *uri;
 
+	session = em_folder_tree_model_get_session (model);
+
 	si = g_hash_table_lookup (model->priv->account_index, account);
 	if (si == NULL)
 		return;
@@ -218,7 +223,7 @@ account_changed_cb (EAccountList *accounts,
 	if (!account->enabled ||!(uri = account->source->url))
 		return;
 
-	if (!(provider = camel_provider_get(uri, NULL)))
+	if (!(provider = camel_provider_get (uri, NULL)))
 		return;
 
 	/* make sure the new store belongs in the tree */
@@ -226,7 +231,8 @@ account_changed_cb (EAccountList *accounts,
 		return;
 
 	store = (CamelStore *) camel_session_get_service (
-		session, uri, CAMEL_PROVIDER_STORE, NULL);
+		CAMEL_SESSION (session), uri,
+		CAMEL_PROVIDER_STORE, NULL);
 	if (store == NULL)
 		return;
 
@@ -249,28 +255,27 @@ account_removed_cb (EAccountList *accounts,
 }
 
 /* HACK: FIXME: the component should listen to the account object directly */
-static void
-add_new_store (gchar *uri,
-               CamelStore *store,
-               gpointer user_data)
-{
-	EAccount *account = user_data;
-
-	if (store == NULL)
-		return;
-
-	e_mail_store_add (store, account->name);
-}
-
 static void
 account_added_cb (EAccountList *accounts,
                   EAccount *account,
                   EMFolderTreeModel *model)
 {
+	EMailSession *session;
+	CamelStore *store;
 	const gchar *uri;
 
+	session = em_folder_tree_model_get_session (model);
 	uri = e_account_get_string (account, E_ACCOUNT_SOURCE_URL);
-	mail_get_store (uri, NULL, add_new_store, account);
+
+	store = (CamelStore *) camel_session_get_service (
+		CAMEL_SESSION (session), uri, CAMEL_PROVIDER_STORE, NULL);
+
+	if (store != NULL) {
+		if ((CAMEL_SERVICE (store)->provider->flags & CAMEL_PROVIDER_IS_STORAGE) != 0)
+			e_mail_store_add (session, store, account->name);
+
+		g_object_unref (store);
+	}
 }
 
 static void
@@ -293,6 +298,12 @@ folder_tree_model_set_property (GObject *object,
 				EM_FOLDER_TREE_MODEL (object),
 				g_value_get_object (value));
 			return;
+
+		case PROP_SESSION:
+			em_folder_tree_model_set_session (
+				EM_FOLDER_TREE_MODEL (object),
+				g_value_get_object (value));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -311,6 +322,13 @@ folder_tree_model_get_property (GObject *object,
 				em_folder_tree_model_get_selection (
 				EM_FOLDER_TREE_MODEL (object)));
 			return;
+
+		case PROP_SESSION:
+			g_value_set_object (
+				value,
+				em_folder_tree_model_get_session (
+				EM_FOLDER_TREE_MODEL (object)));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -330,6 +348,11 @@ folder_tree_model_dispose (GObject *object)
 		priv->selection = NULL;
 	}
 
+	if (priv->session != NULL) {
+		g_object_unref (priv->session);
+		priv->session = NULL;
+	}
+
 	/* Chain up to parent's dispose() method. */
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -357,7 +380,7 @@ folder_tree_model_finalize (GObject *object)
 }
 
 static void
-folder_tree_model_class_init (EMFolderTreeModelClass *class)
+em_folder_tree_model_class_init (EMFolderTreeModelClass *class)
 {
 	GObjectClass *object_class;
 
@@ -380,6 +403,16 @@ folder_tree_model_class_init (EMFolderTreeModelClass *class)
 			GTK_TYPE_TREE_SELECTION,
 			G_PARAM_READWRITE));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_SESSION,
+		g_param_spec_object (
+			"session",
+			NULL,
+			NULL,
+			E_TYPE_MAIL_SESSION,
+			G_PARAM_READWRITE));
+
 	signals[LOADING_ROW] = g_signal_new (
 		"loading-row",
 		G_OBJECT_CLASS_TYPE (object_class),
@@ -470,7 +503,7 @@ folder_tree_model_set_unread_count (EMFolderTreeModel *model,
 }
 
 static void
-folder_tree_model_init (EMFolderTreeModel *model)
+em_folder_tree_model_init (EMFolderTreeModel *model)
 {
 	GHashTable *store_index;
 	GHashTable *uri_index;
@@ -526,38 +559,6 @@ folder_tree_model_init (EMFolderTreeModel *model)
 	model->priv->account_added_id = g_signal_connect (
 		model->priv->accounts, "account-added",
 		G_CALLBACK (account_added_cb), model);
-
-	g_signal_connect_swapped (
-		mail_folder_cache_get_default (),
-		"folder-unread-updated",
-		G_CALLBACK (folder_tree_model_set_unread_count), model);
-}
-
-GType
-em_folder_tree_model_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0)) {
-		static const GTypeInfo type_info = {
-			sizeof (EMFolderTreeModelClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) folder_tree_model_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,  /* class_data */
-			sizeof (EMFolderTreeModel),
-			0,     /* n_preallocs */
-			(GInstanceInitFunc) folder_tree_model_init,
-			NULL   /* value_table */
-		};
-
-		type = g_type_register_static (
-			GTK_TYPE_TREE_STORE, "EMFolderTreeModel",
-			&type_info, 0);
-	}
-
-	return type;
 }
 
 EMFolderTreeModel *
@@ -611,6 +612,47 @@ em_folder_tree_model_set_selection (EMFolderTreeModel *model,
 	g_object_notify (G_OBJECT (model), "selection");
 }
 
+EMailSession *
+em_folder_tree_model_get_session (EMFolderTreeModel *model)
+{
+	g_return_val_if_fail (EM_IS_FOLDER_TREE_MODEL (model), NULL);
+
+	return model->priv->session;
+}
+
+void
+em_folder_tree_model_set_session (EMFolderTreeModel *model,
+                                  EMailSession *session)
+{
+	g_return_if_fail (EM_IS_FOLDER_TREE_MODEL (model));
+
+	if (session != NULL) {
+		g_return_if_fail (E_IS_MAIL_SESSION (session));
+		g_object_ref (session);
+	}
+
+	if (model->priv->session != NULL)
+		g_object_unref (model->priv->session);
+
+	model->priv->session = session;
+
+	/* FIXME Technically we should be disconnecting this signal
+	 *       when replacing an old session with a new session,
+	 *       but at present this function is only called once. */
+	if (session != NULL) {
+		MailFolderCache *folder_cache;
+
+		folder_cache = e_mail_session_get_folder_cache (session);
+
+		g_signal_connect_swapped (
+			folder_cache, "folder-unread-updated",
+			G_CALLBACK (folder_tree_model_set_unread_count),
+			model);
+	}
+
+	g_object_notify (G_OBJECT (model), "session");
+}
+
 void
 em_folder_tree_model_set_folder_info (EMFolderTreeModel *model,
                                       GtkTreeIter *iter,
@@ -620,6 +662,8 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model,
 {
 	GtkTreeRowReference *uri_row, *path_row;
 	GtkTreeStore *tree_store;
+	MailFolderCache *folder_cache;
+	EMailSession *session;
 	guint unread;
 	GtkTreePath *path;
 	GtkTreeIter sub;
@@ -639,6 +683,9 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model,
 
 	tree_store = GTK_TREE_STORE (model);
 
+	session = em_folder_tree_model_get_session (model);
+	folder_cache = e_mail_session_get_folder_cache (session);
+
 	if (!fully_loaded)
 		load = (fi->child == NULL) && !(fi->flags &
 			(CAMEL_FOLDER_NOCHILDREN | CAMEL_FOLDER_NOINFERIORS));
@@ -660,7 +707,7 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model,
 	 *     be functionised. */
 	unread = fi->unread;
 	if (mail_folder_cache_get_folder_from_uri (
-		mail_folder_cache_get_default (), fi->uri, &folder) && folder) {
+		folder_cache, fi->uri, &folder) && folder) {
 		is_drafts = em_utils_folder_is_drafts (folder, fi->uri);
 
 		if (is_drafts || em_utils_folder_is_outbox (folder, fi->uri)) {
@@ -679,8 +726,6 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model,
 		g_object_unref (folder);
 	}
 
-	/* TODO Maybe this should be handled by mail_get_folderinfo
-	 *      (except em-folder-tree doesn't use it, duh) */
 	flags = fi->flags;
 	name = fi->name;
 	if (si->store == e_mail_local_get_store ()) {
@@ -717,7 +762,7 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model,
 			if (camel_store_folder_uri_equal (si->store, fi->uri, curi)) {
 				add_flags = CAMEL_FOLDER_TYPE_SENT;
 			}
-			g_free(curi);
+			g_free (curi);
 		}
 	}
 
@@ -812,9 +857,9 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model,
 }
 
 static void
-folder_subscribed (CamelStore *store,
-                   CamelFolderInfo *fi,
-                   EMFolderTreeModel *model)
+folder_subscribed_cb (CamelStore *store,
+                      CamelFolderInfo *fi,
+                      EMFolderTreeModel *model)
 {
 	EMFolderTreeModelStoreInfo *si;
 	GtkTreeRowReference *reference;
@@ -825,16 +870,16 @@ folder_subscribed (CamelStore *store,
 
 	si = em_folder_tree_model_lookup_store_info (model, store);
 	if (si == NULL)
-		goto done;
+		return;
 
 	/* Make sure we don't already know about it? */
 	if (g_hash_table_lookup (si->full_hash, fi->full_name))
-		goto done;
+		return;
 
 	/* Get our parent folder's path. */
-	dirname = g_alloca(strlen(fi->full_name)+1);
-	strcpy(dirname, fi->full_name);
-	p = strrchr(dirname, '/');
+	dirname = g_alloca (strlen (fi->full_name)+1);
+	strcpy (dirname, fi->full_name);
+	p = strrchr (dirname, '/');
 	if (p == NULL) {
 		/* User subscribed to a toplevel folder. */
 		reference = si->row;
@@ -844,7 +889,7 @@ folder_subscribed (CamelStore *store,
 	}
 
 	if (!gtk_tree_row_reference_valid (reference))
-		goto done;
+		return;
 
 	path = gtk_tree_row_reference_get_path (reference);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &parent, path);
@@ -855,38 +900,19 @@ folder_subscribed (CamelStore *store,
 		GTK_TREE_MODEL (model), &parent,
 		COL_BOOL_LOAD_SUBDIRS, &load, -1);
 	if (load)
-		goto done;
+		return;
 
 	gtk_tree_store_append (GTK_TREE_STORE (model), &iter, &parent);
 
 	em_folder_tree_model_set_folder_info (model, &iter, si, fi, TRUE);
 
 	g_signal_emit (model, signals[FOLDER_ADDED], 0, fi->full_name, fi->uri);
-
-done:
-	g_object_unref (store);
-	camel_folder_info_free (fi);
 }
 
 static void
-folder_subscribed_cb (CamelStore *store,
-                      gpointer event_data,
-                      EMFolderTreeModel *model)
-{
-	CamelFolderInfo *fi;
-
-	g_object_ref (store);
-	fi = camel_folder_info_clone (event_data);
-
-	mail_async_event_emit (
-		mail_async_event, MAIL_ASYNC_GUI, (MailAsyncFunc)
-		folder_subscribed, store, fi, model);
-}
-
-static void
-folder_unsubscribed (CamelStore *store,
-                     CamelFolderInfo *fi,
-                     EMFolderTreeModel *model)
+folder_unsubscribed_cb (CamelStore *store,
+                        CamelFolderInfo *fi,
+                        EMFolderTreeModel *model)
 {
 	EMFolderTreeModelStoreInfo *si;
 	GtkTreeRowReference *reference;
@@ -895,76 +921,50 @@ folder_unsubscribed (CamelStore *store,
 
 	si = em_folder_tree_model_lookup_store_info (model, store);
 	if (si == NULL)
-		goto done;
+		return;
 
 	reference = g_hash_table_lookup (si->full_hash, fi->full_name);
 	if (!gtk_tree_row_reference_valid (reference))
-		goto done;
+		return;
 
 	path = gtk_tree_row_reference_get_path (reference);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
 	gtk_tree_path_free (path);
 
 	em_folder_tree_model_remove_folders (model, si, &iter);
-
-done:
-	g_object_unref (store);
-	camel_folder_info_free (fi);
-}
-
-static void
-folder_unsubscribed_cb (CamelStore *store,
-                        gpointer event_data,
-                        EMFolderTreeModel *model)
-{
-	CamelFolderInfo *fi;
-
-	g_object_ref (store);
-	fi = camel_folder_info_clone (event_data);
-
-	mail_async_event_emit (
-		mail_async_event, MAIL_ASYNC_GUI, (MailAsyncFunc)
-		folder_unsubscribed, store, fi, model);
 }
 
 static void
 folder_created_cb (CamelStore *store,
-                   gpointer event_data,
+                   CamelFolderInfo *fi,
                    EMFolderTreeModel *model)
 {
-	CamelFolderInfo *fi;
+	EMFolderTreeModelStoreInfo *si;
 
 	/* We only want created events to do more
 	 * work if we don't support subscriptions. */
 	if (camel_store_supports_subscriptions (store))
 		return;
 
-	g_object_ref (store);
-	fi = camel_folder_info_clone (event_data);
+	/* process "folder-created" event only when store already loaded */
+	si = em_folder_tree_model_lookup_store_info (model, store);
+	if (si == NULL || g_hash_table_size (si->full_hash) == 0)
+		return;
 
-	mail_async_event_emit (
-		mail_async_event, MAIL_ASYNC_GUI, (MailAsyncFunc)
-		folder_subscribed, store, fi, model);
+	folder_subscribed_cb (store, fi, model);
 }
 
 static void
 folder_deleted_cb (CamelStore *store,
-                   gpointer event_data,
+                   CamelFolderInfo *fi,
                    EMFolderTreeModel *model)
 {
-	CamelFolderInfo *fi;
-
 	/* We only want deleted events to do more
 	 * work if we don't support subscriptions. */
 	if (camel_store_supports_subscriptions (store))
 		return;
 
-	g_object_ref (store);
-	fi = camel_folder_info_clone (event_data);
-
-	mail_async_event_emit (
-		mail_async_event, MAIL_ASYNC_GUI, (MailAsyncFunc)
-		folder_unsubscribed_cb, store, fi, model);
+	folder_unsubscribed_cb (store, fi, model);
 }
 
 typedef struct {
@@ -973,9 +973,10 @@ typedef struct {
 } RenameInfo;
 
 static void
-folder_renamed (CamelStore *store,
-                RenameInfo *info,
-                EMFolderTreeModel *model)
+folder_renamed_cb (CamelStore *store,
+                   const gchar *old_name,
+                   CamelFolderInfo *info,
+                   EMFolderTreeModel *model)
 {
 	EMFolderTreeModelStoreInfo *si;
 	GtkTreeRowReference *reference;
@@ -985,11 +986,11 @@ folder_renamed (CamelStore *store,
 
 	si = em_folder_tree_model_lookup_store_info (model, store);
 	if (si == NULL)
-		goto done;
+		return;
 
-	reference = g_hash_table_lookup (si->full_hash, info->old_base);
+	reference = g_hash_table_lookup (si->full_hash, old_name);
 	if (!gtk_tree_row_reference_valid (reference))
-		goto done;
+		return;
 
 	path = gtk_tree_row_reference_get_path (reference);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
@@ -997,8 +998,8 @@ folder_renamed (CamelStore *store,
 
 	em_folder_tree_model_remove_folders (model, si, &iter);
 
-	parent = g_strdup (info->new->full_name);
-	p = strrchr(parent, '/');
+	parent = g_strdup (info->full_name);
+	p = strrchr (parent, '/');
 	if (p)
 		*p = 0;
 	if (p == NULL || parent == p)
@@ -1010,40 +1011,14 @@ folder_renamed (CamelStore *store,
 	g_free (parent);
 
 	if (!gtk_tree_row_reference_valid (reference))
-		goto done;
+		return;
 
 	path = gtk_tree_row_reference_get_path (reference);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &root, path);
 	gtk_tree_path_free (path);
 
 	gtk_tree_store_append (GTK_TREE_STORE (model), &iter, &root);
-	em_folder_tree_model_set_folder_info (model, &iter, si, info->new, TRUE);
-
-done:
-	g_object_unref (store);
-
-	g_free (info->old_base);
-	camel_folder_info_free (info->new);
-	g_free (info);
-}
-
-static void
-folder_renamed_cb (CamelStore *store,
-                   const gchar *old_name,
-                   CamelFolderInfo *info,
-                   EMFolderTreeModel *model)
-{
-	RenameInfo *rinfo;
-
-	g_object_ref (store);
-
-	rinfo = g_new0 (RenameInfo, 1);
-	rinfo->old_base = g_strdup (old_name);
-	rinfo->new = camel_folder_info_clone (info);
-
-	mail_async_event_emit (
-		mail_async_event, MAIL_ASYNC_GUI, (MailAsyncFunc)
-		folder_renamed, store, rinfo, model);
+	em_folder_tree_model_set_folder_info (model, &iter, si, info, TRUE);
 }
 
 void
@@ -1072,7 +1047,7 @@ em_folder_tree_model_add_store (EMFolderTreeModel *model,
 	uri = camel_url_to_string (
 		((CamelService *) store)->url, CAMEL_URL_HIDE_ALL);
 
-	account = mail_config_get_account_by_source_url (uri);
+	account = e_get_account_by_source_url (uri);
 
 	/* Add the store to the tree. */
 	gtk_tree_store_append (tree_store, &iter, NULL);
