@@ -52,7 +52,6 @@
 #include "mail-mt.h"
 #include "mail-ops.h"
 #include "mail-tools.h"
-#include "mail-config.h"
 #include "mail-send-recv.h"
 #include "mail-vfolder.h"
 
@@ -66,9 +65,6 @@
 #include "e-mail-local.h"
 #include "e-mail-session.h"
 
-/* backward-compatibility cruft */
-#include "e-util/gtk-compat.h"
-
 #define d(x)
 
 #define EM_FOLDER_TREE_GET_PRIVATE(obj) \
@@ -115,6 +111,7 @@ struct _EMFolderTreePrivate {
 	gboolean skip_double_click;
 
 	GtkCellRenderer *text_renderer;
+	PangoEllipsizeMode ellipsize;
 
 	GtkWidget *selectable; /* an ESelectable, where to pass selectable calls */
 
@@ -124,6 +121,9 @@ struct _EMFolderTreePrivate {
 
 enum {
 	PROP_0,
+	PROP_COPY_TARGET_LIST,
+	PROP_ELLIPSIZE,
+	PROP_PASTE_TARGET_LIST,
 	PROP_SESSION
 };
 
@@ -203,20 +203,24 @@ folder_tree_get_folder_info__desc (struct _EMFolderTreeGetFolderInfo *m)
 }
 
 static void
-folder_tree_get_folder_info__exec (struct _EMFolderTreeGetFolderInfo *m)
+folder_tree_get_folder_info__exec (struct _EMFolderTreeGetFolderInfo *m,
+                                   GCancellable *cancellable,
+                                   GError **error)
 {
 	guint32 flags = m->flags | CAMEL_STORE_FOLDER_INFO_SUBSCRIBED;
+	GError *local_error = NULL;
 
 	m->fi = camel_store_get_folder_info_sync (
-		m->store, m->top, flags,
-		m->base.cancellable, &m->base.error);
+		m->store, m->top, flags, cancellable, &local_error);
 
 	/* XXX POP3 stores always return an error because they have
 	 *     no folder hierarchy to scan.  Clear that error so the
 	 *     user doesn't see it. */
-	if (g_error_matches (m->base.error,
+	if (g_error_matches (local_error,
 		CAMEL_STORE_ERROR, CAMEL_STORE_ERROR_NO_FOLDER))
-		g_clear_error (&m->base.error);
+		g_error_free (local_error);
+	else if (local_error != NULL)
+		g_propagate_error (error, local_error);
 }
 
 static void
@@ -652,8 +656,9 @@ folder_tree_cell_edited_cb (EMFolderTree *folder_tree,
 		e_alert_run_dialog_for_args (
 			parent, "mail:no-rename-folder",
 			old_full_name, new_full_name,
-			local_error->message, NULL);
-		g_clear_error (&local_error);
+			local_error ? local_error->message : _("Unknown error"), NULL);
+		if (local_error)
+			g_clear_error (&local_error);
 		goto exit;
 	}
 
@@ -718,6 +723,36 @@ folder_tree_set_session (EMFolderTree *folder_tree,
 	folder_tree->priv->session = g_object_ref (session);
 }
 
+static GtkTargetList *
+folder_tree_get_copy_target_list (EMFolderTree *folder_tree)
+{
+	GtkTargetList *target_list = NULL;
+
+	if (E_IS_SELECTABLE (folder_tree->priv->selectable)) {
+		ESelectable *selectable;
+
+		selectable = E_SELECTABLE (folder_tree->priv->selectable);
+		target_list = e_selectable_get_copy_target_list (selectable);
+	}
+
+	return target_list;
+}
+
+static GtkTargetList *
+folder_tree_get_paste_target_list (EMFolderTree *folder_tree)
+{
+	GtkTargetList *target_list = NULL;
+
+	if (E_IS_SELECTABLE (folder_tree->priv->selectable)) {
+		ESelectable *selectable;
+
+		selectable = E_SELECTABLE (folder_tree->priv->selectable);
+		target_list = e_selectable_get_paste_target_list (selectable);
+	}
+
+	return target_list;
+}
+
 static void
 folder_tree_set_property (GObject *object,
                           guint property_id,
@@ -725,6 +760,12 @@ folder_tree_set_property (GObject *object,
                           GParamSpec *pspec)
 {
 	switch (property_id) {
+		case PROP_ELLIPSIZE:
+			em_folder_tree_set_ellipsize (
+				EM_FOLDER_TREE (object),
+				g_value_get_enum (value));
+			return;
+
 		case PROP_SESSION:
 			folder_tree_set_session (
 				EM_FOLDER_TREE (object),
@@ -742,6 +783,27 @@ folder_tree_get_property (GObject *object,
                           GParamSpec *pspec)
 {
 	switch (property_id) {
+		case PROP_COPY_TARGET_LIST:
+			g_value_set_boxed (
+				value,
+				folder_tree_get_copy_target_list (
+				EM_FOLDER_TREE (object)));
+			return;
+
+		case PROP_ELLIPSIZE:
+			g_value_set_enum (
+				value,
+				em_folder_tree_get_ellipsize (
+				EM_FOLDER_TREE (object)));
+			return;
+
+		case PROP_PASTE_TARGET_LIST:
+			g_value_set_boxed (
+				value,
+				folder_tree_get_paste_target_list (
+				EM_FOLDER_TREE (object)));
+			return;
+
 		case PROP_SESSION:
 			g_value_set_object (
 				value,
@@ -1034,6 +1096,29 @@ folder_tree_class_init (EMFolderTreeClass *class)
 	tree_view_class->test_collapse_row = folder_tree_test_collapse_row;
 	tree_view_class->row_expanded = folder_tree_row_expanded;
 
+	/* Inherited from ESelectableInterface */
+	g_object_class_override_property (
+		object_class,
+		PROP_COPY_TARGET_LIST,
+		"copy-target-list");
+
+	g_object_class_install_property (
+		object_class,
+		PROP_ELLIPSIZE,
+		g_param_spec_enum (
+			"ellipsize",
+			NULL,
+			NULL,
+			PANGO_TYPE_ELLIPSIZE_MODE,
+			PANGO_ELLIPSIZE_NONE,
+			G_PARAM_READWRITE));
+
+	/* Inherited from ESelectableInterface */
+	g_object_class_override_property (
+		object_class,
+		PROP_PASTE_TARGET_LIST,
+		"paste-target-list");
+
 	g_object_class_install_property (
 		object_class,
 		PROP_SESSION,
@@ -1258,10 +1343,6 @@ folder_tree_new (EMFolderTree *folder_tree)
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 	GtkWidget *tree;
-	GConfClient *gconf;
-	const gchar *key;
-
-	gconf = mail_config_get_gconf_client ();
 
 	/* FIXME Gross hack */
 	tree = GTK_WIDGET (folder_tree);
@@ -1281,15 +1362,16 @@ folder_tree_new (EMFolderTree *folder_tree)
 		render_icon, NULL, NULL);
 
 	renderer = gtk_cell_renderer_text_new ();
-	key = "/apps/evolution/mail/display/no_folder_dots";
-	if (!gconf_client_get_bool (gconf, key, NULL))
-		g_object_set (
-			renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_set_cell_data_func (
 		column, renderer, render_display_name, NULL, NULL);
 	folder_tree->priv->text_renderer = g_object_ref (renderer);
 
+	g_object_bind_property (
+		folder_tree, "ellipsize",
+		renderer, "ellipsize",
+		G_BINDING_SYNC_CREATE);
+
 	g_signal_connect_swapped (
 		renderer, "edited",
 		G_CALLBACK (folder_tree_cell_edited_cb), folder_tree);
@@ -1411,7 +1493,9 @@ void
 em_folder_tree_set_selectable_widget (EMFolderTree *folder_tree,
                                       GtkWidget *selectable)
 {
-	if (selectable)
+	g_return_if_fail (EM_IS_FOLDER_TREE (folder_tree));
+
+	if (selectable != NULL)
 		g_return_if_fail (E_IS_SELECTABLE (selectable));
 
 	folder_tree->priv->selectable = selectable;
@@ -1442,40 +1526,125 @@ folder_tree_selectable_update_actions (ESelectable *selectable,
 	}
 }
 
-#define folder_tree_selectable_func(_func)				\
-static void								\
-folder_tree_selectable_ ## _func (ESelectable *selectable)		\
-{									\
-	ESelectableInterface *interface;				\
-	EMFolderTree *folder_tree;					\
-									\
-									\
-	folder_tree = EM_FOLDER_TREE (selectable);			\
-	g_return_if_fail (folder_tree != NULL);				\
-									\
-	if (folder_tree->priv->selectable == NULL)			\
-		return;							\
-									\
-	selectable = E_SELECTABLE (folder_tree->priv->selectable);	\
-	interface = E_SELECTABLE_GET_INTERFACE (selectable);		\
-									\
-	if (interface-> _func == NULL)					\
-		return;							\
-									\
-	if (gtk_widget_get_can_focus (GTK_WIDGET (selectable)) &&	\
-	    !gtk_widget_has_focus (GTK_WIDGET (selectable)))		\
-		gtk_widget_grab_focus (GTK_WIDGET (selectable));	\
-									\
-	interface-> _func (selectable);					\
+static void
+folder_tree_selectable_cut_clipboard (ESelectable *selectable)
+{
+	ESelectableInterface *interface;
+	EMFolderTree *folder_tree;
+	GtkWidget *proxy;
+
+	folder_tree = EM_FOLDER_TREE (selectable);
+	proxy = folder_tree->priv->selectable;
+
+	if (!E_IS_SELECTABLE (proxy))
+		return;
+
+	interface = E_SELECTABLE_GET_INTERFACE (proxy);
+
+	if (interface->cut_clipboard == NULL)
+		return;
+
+	if (gtk_widget_get_can_focus (proxy))
+		gtk_widget_grab_focus (proxy);
+
+	interface->cut_clipboard (E_SELECTABLE (proxy));
 }
 
-folder_tree_selectable_func (cut_clipboard);
-folder_tree_selectable_func (copy_clipboard);
-folder_tree_selectable_func (paste_clipboard);
-folder_tree_selectable_func (delete_selection);
-folder_tree_selectable_func (select_all);
+static void
+folder_tree_selectable_copy_clipboard (ESelectable *selectable)
+{
+	ESelectableInterface *interface;
+	EMFolderTree *folder_tree;
+	GtkWidget *proxy;
 
-#undef folder_tree_selectable_func
+	folder_tree = EM_FOLDER_TREE (selectable);
+	proxy = folder_tree->priv->selectable;
+
+	if (!E_IS_SELECTABLE (proxy))
+		return;
+
+	interface = E_SELECTABLE_GET_INTERFACE (proxy);
+
+	if (interface->copy_clipboard == NULL)
+		return;
+
+	if (gtk_widget_get_can_focus (proxy))
+		gtk_widget_grab_focus (proxy);
+
+	interface->copy_clipboard (E_SELECTABLE (proxy));
+}
+
+static void
+folder_tree_selectable_paste_clipboard (ESelectable *selectable)
+{
+	ESelectableInterface *interface;
+	EMFolderTree *folder_tree;
+	GtkWidget *proxy;
+
+	folder_tree = EM_FOLDER_TREE (selectable);
+	proxy = folder_tree->priv->selectable;
+
+	if (!E_IS_SELECTABLE (proxy))
+		return;
+
+	interface = E_SELECTABLE_GET_INTERFACE (proxy);
+
+	if (interface->paste_clipboard == NULL)
+		return;
+
+	if (gtk_widget_get_can_focus (proxy))
+		gtk_widget_grab_focus (proxy);
+
+	interface->paste_clipboard (E_SELECTABLE (proxy));
+}
+
+static void
+folder_tree_selectable_delete_selection (ESelectable *selectable)
+{
+	ESelectableInterface *interface;
+	EMFolderTree *folder_tree;
+	GtkWidget *proxy;
+
+	folder_tree = EM_FOLDER_TREE (selectable);
+	proxy = folder_tree->priv->selectable;
+
+	if (!E_IS_SELECTABLE (proxy))
+		return;
+
+	interface = E_SELECTABLE_GET_INTERFACE (proxy);
+
+	if (interface->delete_selection == NULL)
+		return;
+
+	if (gtk_widget_get_can_focus (proxy))
+		gtk_widget_grab_focus (proxy);
+
+	interface->delete_selection (E_SELECTABLE (proxy));
+}
+
+static void
+folder_tree_selectable_select_all (ESelectable *selectable)
+{
+	ESelectableInterface *interface;
+	EMFolderTree *folder_tree;
+	GtkWidget *proxy;
+
+	folder_tree = EM_FOLDER_TREE (selectable);
+	proxy = folder_tree->priv->selectable;
+
+	if (!E_IS_SELECTABLE (proxy))
+		return;
+
+	interface = E_SELECTABLE_GET_INTERFACE (proxy);
+
+	if (interface->select_all == NULL)
+		return;
+
+	if (gtk_widget_get_can_focus (proxy))
+		gtk_widget_grab_focus (proxy);
+
+	interface->select_all (E_SELECTABLE (proxy));
+}
 
 static void
 folder_tree_selectable_init (ESelectableInterface *interface)
@@ -1532,6 +1701,28 @@ em_folder_tree_new (EMailSession *session)
 		EM_TYPE_FOLDER_TREE, "session", session, NULL);
 }
 
+PangoEllipsizeMode
+em_folder_tree_get_ellipsize (EMFolderTree *folder_tree)
+{
+	g_return_val_if_fail (EM_IS_FOLDER_TREE (folder_tree), 0);
+
+	return folder_tree->priv->ellipsize;
+}
+
+void
+em_folder_tree_set_ellipsize (EMFolderTree *folder_tree,
+                              PangoEllipsizeMode ellipsize)
+{
+	g_return_if_fail (EM_IS_FOLDER_TREE (folder_tree));
+
+	if (ellipsize == folder_tree->priv->ellipsize)
+		return;
+
+	folder_tree->priv->ellipsize = ellipsize;
+
+	g_object_notify (G_OBJECT (folder_tree), "ellipsize");
+}
+
 EMailSession *
 em_folder_tree_get_session (EMFolderTree *folder_tree)
 {
@@ -1548,8 +1739,7 @@ tree_drag_begin (GtkWidget *widget,
 	EMFolderTreePrivate *priv = folder_tree->priv;
 	GtkTreeSelection *selection;
 	GtkTreeView *tree_view;
-	GdkColormap *colormap;
-	GdkPixmap *pixmap;
+	cairo_surface_t *s;
 	GtkTreeModel *model;
 	GtkTreePath *path;
 	GtkTreeIter iter;
@@ -1562,9 +1752,8 @@ tree_drag_begin (GtkWidget *widget,
 	path = gtk_tree_model_get_path (model, &iter);
 	priv->drag_row = gtk_tree_row_reference_new (model, path);
 
-	pixmap = gtk_tree_view_create_row_drag_icon (tree_view, path);
-	colormap = gdk_drawable_get_colormap (pixmap);
-	gtk_drag_set_icon_pixmap (context, colormap, pixmap, NULL, 0, 0);
+	s = gtk_tree_view_create_row_drag_icon (tree_view, path);
+	gtk_drag_set_icon_surface (context, s);
 
 	gtk_tree_path_free (path);
 }
@@ -1697,6 +1886,7 @@ folder_tree_drop_folder (struct _DragDataReceivedAsync *m)
 {
 	CamelFolder *folder;
 	CamelStore *parent_store;
+	GCancellable *cancellable;
 	const gchar *full_name;
 	const guchar *data;
 
@@ -1704,9 +1894,11 @@ folder_tree_drop_folder (struct _DragDataReceivedAsync *m)
 
 	d(printf(" * Drop folder '%s' onto '%s'\n", data, m->full_name));
 
+	cancellable = e_activity_get_cancellable (m->base.activity);
+
 	folder = e_mail_session_uri_to_folder_sync (
 		m->session, (gchar *) data, 0,
-		m->base.cancellable, &m->base.error);
+		cancellable, &m->base.error);
 	if (folder == NULL)
 		return;
 
@@ -1757,7 +1949,9 @@ folder_tree_drop_async__desc (struct _DragDataReceivedAsync *m)
 }
 
 static void
-folder_tree_drop_async__exec (struct _DragDataReceivedAsync *m)
+folder_tree_drop_async__exec (struct _DragDataReceivedAsync *m,
+                              GCancellable *cancellable,
+                              GError **error)
 {
 	CamelFolder *folder;
 
@@ -1767,19 +1961,18 @@ folder_tree_drop_async__exec (struct _DragDataReceivedAsync *m)
 		folder_tree_drop_folder (m);
 	} else if (m->full_name == NULL) {
 		g_set_error (
-			&m->base.error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot drop message(s) into toplevel store"));
 	} else if ((folder = camel_store_get_folder_sync (
-		m->store, m->full_name, 0,
-		m->base.cancellable, &m->base.error))) {
+		m->store, m->full_name, 0, cancellable, error))) {
 
 		switch (m->info) {
 		case DND_DROP_TYPE_UID_LIST:
 			/* import a list of uids from another evo folder */
 			em_utils_selection_get_uidlist (
 				m->selection, m->session, folder, m->move,
-				m->base.cancellable, &m->base.error);
-			m->moved = m->move && (m->base.error == NULL);
+				cancellable, error);
+			m->moved = m->move && (!error || !*error);
 			break;
 		case DND_DROP_TYPE_MESSAGE_RFC822:
 			/* import a message/rfc822 stream */
@@ -2546,7 +2739,7 @@ em_folder_tree_set_selected_list (EMFolderTree *folder_tree,
 			/* This makes sure all our parents up to the root are
 			 * expanded.  FIXME: Why does the expanded state store
 			 * this made up path rather than the euri? */
-			if ((account = mail_config_get_account_by_source_url (u->uri)))
+			if ((account = e_get_account_by_source_url (u->uri)))
 				expand_key = g_strdup_printf ("%s/%s", account->uid, path);
 			else if (CAMEL_IS_VEE_STORE (u->store))
 				expand_key = g_strdup_printf ("vfolder/%s", path);
@@ -3082,7 +3275,7 @@ em_folder_tree_restore_state (EMFolderTree *folder_tree,
 		/* Do not expand local stores in Express mode. */
 		if (e_shell_get_express_mode (shell)) {
 			expand_row &= (strncmp (uri, "vfolder", 7) != 0);
-			expand_row &= (strncmp (uri, "mbox", 4) != 0);
+			expand_row &= (strncmp (uri, "maildir", 7) != 0);
 		}
 
 		if (expand_row) {
