@@ -40,12 +40,9 @@
 #include "e-mail-local.h"
 #include "mail-config.h"
 #include "mail-folder-cache.h"
-#include "mail-session.h"
 #include "mail-tools.h"
 
 typedef struct {
-	GConfClient *gconf;
-
 	gchar *gtkrc;
 
 	GSList *labels;
@@ -81,7 +78,7 @@ config_write_style (void)
 		return;
 	}
 
-	client = config->gconf;
+	client = gconf_client_get_default ();
 
 	key = "/apps/evolution/mail/display/fonts/use_custom";
 	custom = gconf_client_get_bool (client, key, NULL);
@@ -102,7 +99,8 @@ config_write_style (void)
 	fprintf (rc, "        GtkHTML::spell_error_color = \"%s\"\n", spell_color);
 	g_free (spell_color);
 
-	if (gconf_client_get_bool (config->gconf, "/apps/evolution/mail/display/mark_citations", NULL))
+	key = "/apps/evolution/mail/display/mark_citations";
+	if (gconf_client_get_bool (client, key, NULL))
 		fprintf (rc, "        GtkHTML::cite_color = \"%s\"\n",
 			 citation_color);
 	g_free (citation_color);
@@ -123,32 +121,43 @@ config_write_style (void)
 	fclose (rc);
 
 	gtk_rc_reparse_all ();
+
+	g_object_unref (client);
 }
 
 static void
-gconf_style_changed (GConfClient *client, guint cnxn_id,
-		     GConfEntry *entry, gpointer user_data)
+gconf_style_changed (GConfClient *client,
+                     guint cnxn_id,
+                     GConfEntry *entry,
+                     gpointer user_data)
 {
 	config_write_style ();
 }
 
 static void
-gconf_outlook_filenames_changed (GConfClient *client, guint cnxn_id,
-				 GConfEntry *entry, gpointer user_data)
+gconf_outlook_filenames_changed (GConfClient *client,
+                                 guint cnxn_id,
+                                 GConfEntry *entry,
+                                 gpointer user_data)
 {
+	const gchar *key;
+
 	g_return_if_fail (client != NULL);
 
+	key = "/apps/evolution/mail/composer/outlook_filenames";
+
 	/* pass option to the camel */
-	if (gconf_client_get_bool (client, "/apps/evolution/mail/composer/outlook_filenames", NULL)) {
+	if (gconf_client_get_bool (client, key, NULL))
 		camel_header_param_encode_filenames_in_rfc_2047 = 1;
-	} else {
+	else
 		camel_header_param_encode_filenames_in_rfc_2047 = 0;
-	}
 }
 
 static void
-gconf_jh_headers_changed (GConfClient *client, guint cnxn_id,
-                          GConfEntry *entry, gpointer user_data)
+gconf_jh_headers_changed (GConfClient *client,
+                          guint cnxn_id,
+                          GConfEntry *entry,
+                          EMailSession *session)
 {
 	GSList *node;
 	GPtrArray *name, *value;
@@ -156,19 +165,24 @@ gconf_jh_headers_changed (GConfClient *client, guint cnxn_id,
 	g_slist_foreach (config->jh_header, (GFunc) g_free, NULL);
 	g_slist_free (config->jh_header);
 
-	config->jh_header = gconf_client_get_list (config->gconf, "/apps/evolution/mail/junk/custom_header", GCONF_VALUE_STRING, NULL);
+	config->jh_header = gconf_client_get_list (
+		client, "/apps/evolution/mail/junk/custom_header",
+		GCONF_VALUE_STRING, NULL);
 
 	node = config->jh_header;
 	name = g_ptr_array_new ();
 	value = g_ptr_array_new ();
 	while (node && node->data) {
 		gchar **tok = g_strsplit (node->data, "=", 2);
-		g_ptr_array_add (name, g_strdup(tok[0]));
-		g_ptr_array_add (value, g_strdup(tok[1]));
+		g_ptr_array_add (name, g_strdup (tok[0]));
+		g_ptr_array_add (value, g_strdup (tok[1]));
 		node = node->next;
 		g_strfreev (tok);
 	}
-	mail_session_set_junk_headers ((const gchar **)name->pdata, (const gchar **)value->pdata, name->len);
+	camel_session_set_junk_headers (
+		CAMEL_SESSION (session),
+		(const gchar **) name->pdata,
+		(const gchar **) value->pdata, name->len);
 
 	g_ptr_array_foreach (name, (GFunc) g_free, NULL);
 	g_ptr_array_foreach (value, (GFunc) g_free, NULL);
@@ -177,14 +191,18 @@ gconf_jh_headers_changed (GConfClient *client, guint cnxn_id,
 }
 
 static void
-gconf_jh_check_changed (GConfClient *client, guint cnxn_id,
-		     GConfEntry *entry, gpointer user_data)
+gconf_jh_check_changed (GConfClient *client,
+                        guint cnxn_id,
+                        GConfEntry *entry,
+                        EMailSession *session)
 {
-	config->jh_check = gconf_client_get_bool (config->gconf, "/apps/evolution/mail/junk/check_custom_header", NULL);
+	config->jh_check = gconf_client_get_bool (
+		client, "/apps/evolution/mail/junk/check_custom_header", NULL);
 	if (!config->jh_check) {
-		mail_session_set_junk_headers (NULL, NULL, 0);
+		camel_session_set_junk_headers (
+			CAMEL_SESSION (session), NULL, NULL, 0);
 	} else {
-		gconf_jh_headers_changed (NULL, 0, NULL, NULL);
+		gconf_jh_headers_changed (client, 0, NULL, session);
 	}
 }
 
@@ -221,6 +239,7 @@ gconf_int_value_changed (GConfClient *client,
 void
 mail_config_write (void)
 {
+	GConfClient *client;
 	EAccountList *account_list;
 	ESignatureList *signature_list;
 
@@ -233,17 +252,9 @@ mail_config_write (void)
 	e_account_list_save (account_list);
 	e_signature_list_save (signature_list);
 
-	gconf_client_suggest_sync (config->gconf, NULL);
-}
-
-/* Accessor functions */
-GConfClient *
-mail_config_get_gconf_client (void)
-{
-	if (!config)
-		mail_config_init ();
-
-	return config->gconf;
+	client = gconf_client_get_default ();
+	gconf_client_suggest_sync (client, NULL);
+	g_object_unref (client);
 }
 
 gint
@@ -259,140 +270,27 @@ mail_config_get_address_count (void)
 gint
 mail_config_get_sync_timeout (void)
 {
-	GConfClient *gconf = mail_config_get_gconf_client ();
+	GConfClient *client;
 	gint res = 60;
+	GError *error = NULL;
 
-	if (gconf) {
-		GError *error = NULL;
+	client = gconf_client_get_default ();
 
-		res = gconf_client_get_int (gconf, "/apps/evolution/mail/sync_interval", &error);
+	res = gconf_client_get_int (
+		client, "/apps/evolution/mail/sync_interval", &error);
 
-		/* do not allow recheck sooner than every 30 seconds */
-		if (error || res == 0)
-			res = 60;
-		else if (res < 30)
-			res = 30;
+	/* do not allow recheck sooner than every 30 seconds */
+	if (error || res == 0)
+		res = 60;
+	else if (res < 30)
+		res = 30;
 
-		if (error)
-			g_error_free (error);
-	}
+	if (error)
+		g_error_free (error);
 
 	return res;
 }
 
-static EAccount *
-mc_get_account_by (const gchar *given_url, const gchar * (get_url_string)(EAccount *account))
-{
-	EAccountList *account_list;
-	EAccount *account = NULL;
-	EIterator *iter;
-	CamelURL *url;
-	CamelProvider *provider;
-
-	g_return_val_if_fail (given_url != NULL, NULL);
-	g_return_val_if_fail (get_url_string != NULL, NULL);
-
-	url = camel_url_new (given_url, NULL);
-	g_return_val_if_fail (url != NULL, NULL);
-
-	provider = camel_provider_get (given_url, NULL);
-	g_return_val_if_fail (provider != NULL && provider->url_equal != NULL, NULL);
-
-	account_list = e_get_account_list ();
-	iter = e_list_get_iterator ((EList *) account_list);
-	while (account == NULL && e_iterator_is_valid (iter)) {
-		CamelURL *account_url;
-		const gchar *account_url_string;
-
-		account = (EAccount *) e_iterator_get (iter);
-
-		e_iterator_next (iter);
-
-		account_url_string = get_url_string (account);
-
-		if ( !account_url_string || !*account_url_string) {
-			account = NULL;
-			continue;
-		}
-
-		account_url = camel_url_new (account_url_string, NULL);
-		if (account_url == NULL) {
-			account = NULL;
-			continue;
-		}
-
-		if (!provider->url_equal (url, account_url))
-			account = NULL;  /* not a match */
-
-		camel_url_free (account_url);
-	}
-
-	g_object_unref (iter);
-	camel_url_free (url);
-
-	return account;
-}
-
-static const gchar *
-get_source_url_string (EAccount *account)
-{
-	if (account && account->source && account->source->url && *account->source->url)
-		return account->source->url;
-	return NULL;
-}
-
-static const gchar *
-get_transport_url_string (EAccount *account)
-{
-	if (account && account->transport && account->transport->url && *account->transport->url)
-		return account->transport->url;
-	return NULL;
-}
-
-EAccount *
-mail_config_get_account_by_source_url (const gchar *source_url)
-{
-	return mc_get_account_by (source_url, get_source_url_string);
-}
-
-EAccount *
-mail_config_get_account_by_transport_url (const gchar *transport_url)
-{
-	return mc_get_account_by (transport_url, get_transport_url_string);
-}
-
-EAccountService *
-mail_config_get_default_transport (void)
-{
-	EAccountList *account_list;
-	EAccount *account;
-	EIterator *iter;
-
-	account_list = e_get_account_list ();
-	account = e_get_default_account ();
-
-	if (account && account->enabled && account->transport && account->transport->url && account->transport->url[0])
-		return account->transport;
-
-	/* return the first account with a transport? */
-	iter = e_list_get_iterator ((EList *) account_list);
-	while (e_iterator_is_valid (iter)) {
-		account = (EAccount *) e_iterator_get (iter);
-
-		if (account->enabled && account->transport && account->transport->url && account->transport->url[0]) {
-			g_object_unref (iter);
-
-			return account->transport;
-		}
-
-		e_iterator_next (iter);
-	}
-
-	g_object_unref (iter);
-
-	return NULL;
-}
-
 static gchar *
 uri_to_evname (const gchar *uri, const gchar *prefix)
 {
@@ -480,9 +378,9 @@ mail_config_uri_deleted (GCompareFunc uri_cmp, const gchar *uri)
 
 	/* assumes these can't be removed ... */
 	local_drafts_folder_uri =
-		e_mail_local_get_folder_uri (E_MAIL_FOLDER_DRAFTS);
+		e_mail_local_get_folder_uri (E_MAIL_LOCAL_FOLDER_DRAFTS);
 	local_sent_folder_uri =
-		e_mail_local_get_folder_uri (E_MAIL_FOLDER_SENT);
+		e_mail_local_get_folder_uri (E_MAIL_LOCAL_FOLDER_SENT);
 
 	account_list = e_get_account_list ();
 	iter = e_list_get_iterator ((EList *) account_list);
@@ -509,23 +407,6 @@ mail_config_uri_deleted (GCompareFunc uri_cmp, const gchar *uri)
 		mail_config_write ();
 }
 
-void
-mail_config_service_set_save_passwd (EAccountService *service, gboolean save_passwd)
-{
-	service->save_passwd = save_passwd;
-}
-
-gchar *
-mail_config_folder_to_safe_url (CamelFolder *folder)
-{
-	gchar *url;
-
-	url = mail_tools_folder_to_url (folder);
-	e_filename_make_safe (url);
-
-	return url;
-}
-
 gchar *
 mail_config_folder_to_cachename (CamelFolder *folder, const gchar *prefix)
 {
@@ -533,7 +414,8 @@ mail_config_folder_to_cachename (CamelFolder *folder, const gchar *prefix)
 	const gchar *config_dir;
 
 	config_dir = mail_session_get_config_dir ();
-	url = mail_config_folder_to_safe_url (folder);
+	url = g_strdup (camel_folder_get_uri (folder));
+	e_filename_make_safe (url);
 	basename = g_strdup_printf ("%s%s", prefix, url);
 	filename = g_build_filename (config_dir, "folders", basename, NULL);
 	g_free (basename);
@@ -543,22 +425,26 @@ mail_config_folder_to_cachename (CamelFolder *folder, const gchar *prefix)
 }
 
 void
-mail_config_reload_junk_headers (void)
+mail_config_reload_junk_headers (EMailSession *session)
 {
+	g_return_if_fail (E_IS_MAIL_SESSION (session));
+
 	/* It automatically sets in the session */
 	if (config == NULL)
-		mail_config_init ();
-	else
-		gconf_jh_check_changed (config->gconf, 0, NULL, config);
+		mail_config_init (session);
+	else {
+		GConfClient *client;
 
+		client = gconf_client_get_default ();
+		gconf_jh_check_changed (client, 0, NULL, session);
+		g_object_unref (client);
+	}
 }
 
 gboolean
 mail_config_get_lookup_book (void)
 {
-	/* It automatically sets in the session */
-	if (config == NULL)
-		mail_config_init ();
+	g_return_val_if_fail (config != NULL, FALSE);
 
 	return config->book_lookup;
 }
@@ -566,154 +452,163 @@ mail_config_get_lookup_book (void)
 gboolean
 mail_config_get_lookup_book_local_only (void)
 {
-	/* It automatically sets in the session */
-	if (config == NULL)
-		mail_config_init ();
+	g_return_val_if_fail (config != NULL, FALSE);
 
 	return config->book_lookup_local_only;
 }
 
 static void
-folder_deleted_cb (MailFolderCache *cache, CamelStore *store, const gchar *uri, gpointer user_data)
+folder_deleted_cb (MailFolderCache *cache,
+                   CamelStore *store,
+                   const gchar *uri,
+                   gpointer user_data)
 {
-	mail_config_uri_deleted(CAMEL_STORE_CLASS(CAMEL_OBJECT_GET_CLASS(store))->compare_folder_name, uri);
+	CamelStoreClass *class;
+
+	class = CAMEL_STORE_GET_CLASS (store);
+	mail_config_uri_deleted (class->compare_folder_name, uri);
 }
 
 static void
-folder_renamed_cb (MailFolderCache *cache, CamelStore *store, const gchar *olduri, const gchar *newuri, gpointer user_data)
+folder_renamed_cb (MailFolderCache *cache,
+                   CamelStore *store,
+                   const gchar *olduri,
+                   const gchar *newuri,
+                   gpointer user_data)
 {
-	mail_config_uri_renamed(CAMEL_STORE_CLASS(CAMEL_OBJECT_GET_CLASS(store))->compare_folder_name,
-				olduri, newuri);
+	CamelStoreClass *class;
+
+	class = CAMEL_STORE_GET_CLASS (store);
+	mail_config_uri_renamed (class->compare_folder_name, olduri, newuri);
 }
 
 /* Config struct routines */
 void
-mail_config_init (void)
+mail_config_init (EMailSession *session)
 {
+	GConfClient *client;
 	GConfClientNotifyFunc func;
+	MailFolderCache *folder_cache;
 	const gchar *key;
 
+	g_return_if_fail (E_IS_MAIL_SESSION (session));
+
 	if (config)
 		return;
 
 	config = g_new0 (MailConfig, 1);
-	config->gconf = gconf_client_get_default ();
 	config->gtkrc = g_build_filename (
 		mail_session_get_config_dir (),
 		"gtkrc-mail-fonts", NULL);
 
 	gtk_rc_parse (config->gtkrc);
 
+	client = gconf_client_get_default ();
+
 	gconf_client_add_dir (
-		config->gconf, "/apps/evolution/mail/prompts",
+		client, "/apps/evolution/mail/prompts",
 		GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
 	/* Composer Configuration */
 
 	gconf_client_add_dir (
-		config->gconf, "/apps/evolution/mail/composer",
+		client, "/apps/evolution/mail/composer",
 		GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
 	key = "/apps/evolution/mail/composer/spell_color";
 	func = (GConfClientNotifyFunc) gconf_style_changed;
-	gconf_client_notify_add (
-		config->gconf, key, func, NULL, NULL, NULL);
+	gconf_client_notify_add (client, key, func, NULL, NULL, NULL);
 
 	key = "/apps/evolution/mail/composer/outlook_filenames";
 	func = (GConfClientNotifyFunc) gconf_outlook_filenames_changed;
-	gconf_outlook_filenames_changed (config->gconf, 0, NULL, NULL);
-	gconf_client_notify_add (
-		config->gconf, key, func, NULL, NULL, NULL);
+	gconf_outlook_filenames_changed (client, 0, NULL, NULL);
+	gconf_client_notify_add (client, key, func, NULL, NULL, NULL);
 
 	/* Display Configuration */
 
 	gconf_client_add_dir (
-		config->gconf, "/apps/evolution/mail/display",
+		client, "/apps/evolution/mail/display",
 		GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
 	key = "/apps/evolution/mail/display/address_compress";
 	func = (GConfClientNotifyFunc) gconf_bool_value_changed;
 	gconf_client_notify_add (
-		config->gconf, key, func,
+		client, key, func,
 		&config->address_compress, NULL, NULL);
-	config->address_compress =
-		gconf_client_get_bool (config->gconf, key, NULL);
+	config->address_compress = gconf_client_get_bool (client, key, NULL);
 
 	key = "/apps/evolution/mail/display/address_count";
 	func = (GConfClientNotifyFunc) gconf_int_value_changed;
 	gconf_client_notify_add (
-		config->gconf, key, func,
+		client, key, func,
 		&config->address_count, NULL, NULL);
-	config->address_count =
-		gconf_client_get_int (config->gconf, key, NULL);
+	config->address_count = gconf_client_get_int (client, key, NULL);
 
 	key = "/apps/evolution/mail/display/citation_colour";
 	func = (GConfClientNotifyFunc) gconf_style_changed;
-	gconf_client_notify_add (
-		config->gconf, key, func, NULL, NULL, NULL);
+	gconf_client_notify_add (client, key, func, NULL, NULL, NULL);
 
 	key = "/apps/evolution/mail/display/mark_citations";
 	func = (GConfClientNotifyFunc) gconf_style_changed;
-	gconf_client_notify_add (
-		config->gconf, key, func, NULL, NULL, NULL);
+	gconf_client_notify_add (client, key, func, NULL, NULL, NULL);
 
 	/* Font Configuration */
 
 	gconf_client_add_dir (
-		config->gconf, "/apps/evolution/mail/display/fonts",
+		client, "/apps/evolution/mail/display/fonts",
 		GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
 	key = "/apps/evolution/mail/display/fonts";
 	func = (GConfClientNotifyFunc) gconf_style_changed;
-	gconf_client_notify_add (
-		config->gconf, key, func, NULL, NULL, NULL);
+	gconf_client_notify_add (client, key, func, NULL, NULL, NULL);
 
 	/* Junk Configuration */
 
 	gconf_client_add_dir (
-		config->gconf, "/apps/evolution/mail/junk",
+		client, "/apps/evolution/mail/junk",
 		GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
 	key = "/apps/evolution/mail/junk/check_custom_header";
 	func = (GConfClientNotifyFunc) gconf_jh_check_changed;
-	gconf_client_notify_add (
-		config->gconf, key, func, NULL, NULL, NULL);
-	config->jh_check =
-		gconf_client_get_bool (config->gconf, key, NULL);
+	gconf_client_notify_add (client, key, func, session, NULL, NULL);
+	config->jh_check = gconf_client_get_bool (client, key, NULL);
 
 	key = "/apps/evolution/mail/junk/custom_header";
 	func = (GConfClientNotifyFunc) gconf_jh_headers_changed;
-	gconf_client_notify_add (
-		config->gconf, key, func, NULL, NULL, NULL);
+	gconf_client_notify_add (client, key, func, session, NULL, NULL);
 
 	key = "/apps/evolution/mail/junk/lookup_addressbook";
 	func = (GConfClientNotifyFunc) gconf_bool_value_changed;
 	gconf_client_notify_add (
-		config->gconf, key, func,
+		client, key, func,
 		&config->book_lookup, NULL, NULL);
-	config->book_lookup =
-		gconf_client_get_bool (config->gconf, key, NULL);
+	config->book_lookup = gconf_client_get_bool (client, key, NULL);
 
 	key = "/apps/evolution/mail/junk/lookup_addressbook_local_only";
 	func = (GConfClientNotifyFunc) gconf_bool_value_changed;
 	gconf_client_notify_add (
-		config->gconf, key, func,
+		client, key, func,
 		&config->book_lookup_local_only, NULL, NULL);
 	config->book_lookup_local_only =
-		gconf_client_get_bool (config->gconf, key, NULL);
+		gconf_client_get_bool (client, key, NULL);
 
 	key = "/desktop/gnome/lockdown/disable_command_line";
 	func = (GConfClientNotifyFunc) gconf_bool_value_changed;
 	gconf_client_notify_add (
-		config->gconf, key, func,
+		client, key, func,
 		&config->scripts_disabled, NULL, NULL);
-	config->scripts_disabled =
-		gconf_client_get_bool (config->gconf, key, NULL);
+	config->scripts_disabled = gconf_client_get_bool (client, key, NULL);
 
-	gconf_jh_check_changed (config->gconf, 0, NULL, config);
+	gconf_jh_check_changed (client, 0, NULL, session);
 
-	g_signal_connect (mail_folder_cache_get_default (), "folder-deleted",
-			  (GCallback) folder_deleted_cb, NULL);
-	g_signal_connect (mail_folder_cache_get_default (), "folder-renamed",
-			  (GCallback) folder_renamed_cb, NULL);
+	folder_cache = e_mail_session_get_folder_cache (session);
+
+	g_signal_connect (
+		folder_cache, "folder-deleted",
+		(GCallback) folder_deleted_cb, NULL);
+	g_signal_connect (
+		folder_cache, "folder-renamed",
+		(GCallback) folder_renamed_cb, NULL);
+
+	g_object_unref (client);
 }
