@@ -63,9 +63,13 @@ cal_shell_view_execute_search (EShellView *shell_view)
 	EActionComboBox *combo_box;
 	GnomeCalendar *calendar;
 	ECalendar *date_navigator;
+	ECalModel *model;
 	GtkRadioAction *action;
+	icaltimezone *timezone;
+	struct icaltimetype current_time;
 	time_t start_range;
 	time_t end_range;
+	time_t now_time;
 	gboolean range_search;
 	gchar *start, *end;
 	gchar *query;
@@ -81,6 +85,12 @@ cal_shell_view_execute_search (EShellView *shell_view)
 
 	searchbar = e_cal_shell_content_get_searchbar (cal_shell_content);
 
+	calendar = e_cal_shell_content_get_calendar (cal_shell_content);
+	model = gnome_calendar_get_model (calendar);
+	timezone = e_cal_model_get_timezone (model);
+	current_time = icaltime_current_time_with_zone (timezone);
+	now_time = time_day_begin (icaltime_as_timet (current_time));
+
 	action = GTK_RADIO_ACTION (ACTION (CALENDAR_SEARCH_ANY_FIELD_CONTAINS));
 	value = gtk_radio_action_get_current_value (action);
 
@@ -145,8 +155,8 @@ cal_shell_view_execute_search (EShellView *shell_view)
 
 		case CALENDAR_FILTER_ACTIVE_APPOINTMENTS:
 			/* Show a year's worth of appointments. */
-			start_range = time (NULL);
-			end_range = time_add_day (start_range, 365);
+			start_range = now_time;
+			end_range = time_day_end (time_add_day (start_range, 365));
 			start = isodate_from_time_t (start_range);
 			end = isodate_from_time_t (end_range);
 
@@ -161,8 +171,8 @@ cal_shell_view_execute_search (EShellView *shell_view)
 			break;
 
 		case CALENDAR_FILTER_NEXT_7_DAYS_APPOINTMENTS:
-			start_range = time (NULL);
-			end_range = time_add_day (start_range, 7);
+			start_range = now_time;
+			end_range = time_day_end (time_add_day (start_range, 7));
 			start = isodate_from_time_t (start_range);
 			end = isodate_from_time_t (end_range);
 
@@ -207,7 +217,6 @@ cal_shell_view_execute_search (EShellView *shell_view)
 	}
 
 	/* Submit the query. */
-	calendar = e_cal_shell_content_get_calendar (cal_shell_content);
 	gnome_calendar_set_search_query (
 		calendar, query, range_search, start_range, end_range);
 	g_free (query);
@@ -277,12 +286,14 @@ cal_shell_view_update_actions (EShellView *shell_view)
 	ECalShellContent *cal_shell_content;
 	EShellSidebar *shell_sidebar;
 	EShellWindow *shell_window;
+	EShell *shell;
 	GnomeCalendarViewType view_type;
 	GnomeCalendar *calendar;
 	ECalendarView *view;
 	GtkAction *action;
 	GList *list, *iter;
 	gboolean sensitive;
+	gboolean visible;
 	guint32 state;
 	gint n_selected;
 
@@ -302,40 +313,24 @@ cal_shell_view_update_actions (EShellView *shell_view)
 	priv = E_CAL_SHELL_VIEW_GET_PRIVATE (shell_view);
 
 	shell_window = e_shell_view_get_shell_window (shell_view);
+	shell = e_shell_window_get_shell (shell_window);
 
-	if (e_shell_get_express_mode(e_shell_get_default())) {
-		GtkWidget *widget, *item;
-		GdkScreen *scr;
-		gint monitor;
-		GdkRectangle rect;
+	if (e_shell_get_express_mode (shell)) {
+		GtkWidget *widget;
 
-		/* Hack: Get rid of New and Send/Receive in toolbar
-		 * while in express mode */
+		/* Hide the New button on the toolbar. */
 		widget = e_shell_window_get_managed_widget (
 			shell_window, "/main-toolbar");
+		widget = (GtkWidget *)gtk_toolbar_get_nth_item (
+			GTK_TOOLBAR (widget), 0);
+		gtk_widget_hide (widget);
 
-		item = (GtkWidget *)gtk_toolbar_get_nth_item ((GtkToolbar *)widget, 0);
-		gtk_widget_hide(item);
-
+		/* Hide the main menu. */
 		widget = e_shell_window_get_managed_widget (
 			shell_window, "/main-menu");
-		gtk_widget_hide(widget);
-
-		item = e_shell_window_get_managed_widget (
-			shell_window, "/main-toolbar/send-receive");
-		gtk_widget_hide(item);
-
-		scr = gdk_screen_get_default ();
-		monitor = gdk_screen_get_monitor_at_window (
-			scr, gtk_widget_get_window (GTK_WIDGET (shell_window)));
-		gdk_screen_get_monitor_geometry (scr, monitor, &rect);
-
-		gtk_window_set_default_size (
-			GTK_WINDOW (shell_window), rect.width, rect.height);
-		gtk_window_set_decorated (
-			GTK_WINDOW (shell_window), FALSE);
-
+		gtk_widget_hide (widget);
 	}
+
 	cal_shell_content = priv->cal_shell_content;
 	calendar = e_cal_shell_content_get_calendar (cal_shell_content);
 	view_type = gnome_calendar_get_view (calendar);
@@ -477,6 +472,10 @@ cal_shell_view_update_actions (EShellView *shell_view)
 	action = ACTION (EVENT_REPLY_ALL);
 	sensitive = (n_selected == 1) && is_meeting;
 	gtk_action_set_sensitive (action, sensitive);
+
+	action = ACTION (EVENT_MEETING_NEW);
+	visible = itip_addresses_get_default () != NULL;
+	gtk_action_set_visible (action, visible);
 }
 
 static void
@@ -506,7 +505,10 @@ cal_shell_view_class_init (ECalShellViewClass *class,
 	shell_view_class->execute_search = cal_shell_view_execute_search;
 	shell_view_class->update_actions = cal_shell_view_update_actions;
 
-	if (e_shell_get_express_mode(e_shell_get_default()))
+	/* XXX This is an unusual place to need an EShell instance.
+	 *     Would be cleaner to implement a method that either
+	 *     chains up or does nothing based on express mode. */
+	if (e_shell_get_express_mode (e_shell_get_default ()))
 		shell_view_class->construct_searchbar = NULL;
 
 }
