@@ -28,7 +28,6 @@
 #include "e-util/e-alert-dialog.h"
 #include "e-util/e-util.h"
 #include "calendar/common/authentication.h"
-#include "calendar/gui/calendar-config.h"
 #include "calendar/gui/e-task-list-selector.h"
 #include "calendar/gui/misc.h"
 
@@ -107,7 +106,7 @@ task_shell_sidebar_backend_died_cb (ETaskShellSidebar *task_shell_sidebar,
                                     ECal *client)
 {
 	EShellView *shell_view;
-	EShellWindow *shell_window;
+	EShellContent *shell_content;
 	EShellSidebar *shell_sidebar;
 	GHashTable *client_table;
 	ESource *source;
@@ -117,7 +116,7 @@ task_shell_sidebar_backend_died_cb (ETaskShellSidebar *task_shell_sidebar,
 
 	shell_sidebar = E_SHELL_SIDEBAR (task_shell_sidebar);
 	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
-	shell_window = e_shell_view_get_shell_window (shell_view);
+	shell_content = e_shell_view_get_shell_content (shell_view);
 
 	source = e_cal_get_source (client);
 	uid = e_source_peek_uid (source);
@@ -127,8 +126,8 @@ task_shell_sidebar_backend_died_cb (ETaskShellSidebar *task_shell_sidebar,
 	g_hash_table_remove (client_table, uid);
 	task_shell_sidebar_emit_status_message (task_shell_sidebar, NULL);
 
-	e_alert_run_dialog_for_args (
-		GTK_WINDOW (shell_window),
+	e_alert_submit (
+		E_ALERT_SINK (shell_content),
 		"calendar:tasks-crashed", NULL);
 
 	g_object_unref (source);
@@ -140,35 +139,23 @@ task_shell_sidebar_backend_error_cb (ETaskShellSidebar *task_shell_sidebar,
                                      ECal *client)
 {
 	EShellView *shell_view;
-	EShellWindow *shell_window;
+	EShellContent *shell_content;
 	EShellSidebar *shell_sidebar;
-	GtkWidget *dialog;
-	const gchar *uri;
-	gchar *uri_no_passwd;
+	ESourceGroup *source_group;
+	ESource *source;
 
 	shell_sidebar = E_SHELL_SIDEBAR (task_shell_sidebar);
 	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
-	shell_window = e_shell_view_get_shell_window (shell_view);
+	shell_content = e_shell_view_get_shell_content (shell_view);
 
-	uri = e_cal_get_uri (client);
-	uri_no_passwd = get_uri_without_password (uri);
+	source = e_cal_get_source (client);
+	source_group = e_source_peek_group (source);
 
-	/* Translators: This string is displayed in a message dialog when
-	 *              our connection to the calendar service detects an
-	 *              out-of-band error.  The first string is a URI for
-	 *              the source of the error, the second string is the
-	 *              error message. */
-	dialog = gtk_message_dialog_new (
-		GTK_WINDOW (shell_window),
-		GTK_DIALOG_DESTROY_WITH_PARENT,
-		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-		_("Error on %s\n%s"),
-		uri_no_passwd, message);
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
-
-	g_free (uri_no_passwd);
+	e_alert_submit (
+		E_ALERT_SINK (shell_content),
+		"calendar:backend-error",
+		e_source_group_peek_name (source_group),
+		e_source_peek_name (source), message, NULL);
 }
 
 static void
@@ -177,13 +164,13 @@ task_shell_sidebar_client_opened_cb (ETaskShellSidebar *task_shell_sidebar,
                                      ECal *client)
 {
 	EShellView *shell_view;
-	EShellWindow *shell_window;
+	EShellContent *shell_content;
 	EShellSidebar *shell_sidebar;
 	const gchar *message;
 
 	shell_sidebar = E_SHELL_SIDEBAR (task_shell_sidebar);
 	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
-	shell_window = e_shell_view_get_shell_window (shell_view);
+	shell_content = e_shell_view_get_shell_content (shell_view);
 
 	if (g_error_matches (error, E_CALENDAR_ERROR,
 		E_CALENDAR_STATUS_AUTHENTICATION_FAILED) ||
@@ -204,16 +191,16 @@ task_shell_sidebar_client_opened_cb (ETaskShellSidebar *task_shell_sidebar,
 			return;
 
 		case E_CALENDAR_STATUS_REPOSITORY_OFFLINE:
-			e_alert_run_dialog_for_args (
-				GTK_WINDOW (shell_window),
+			e_alert_submit (
+				E_ALERT_SINK (shell_content),
 				"calendar:prompt-no-contents-offline-tasks",
 				NULL);
 			/* fall through */
 
 		default:
 			if (error->code != E_CALENDAR_STATUS_REPOSITORY_OFFLINE) {
-				e_alert_run_dialog_for_args (
-					GTK_WINDOW (shell_window),
+				e_alert_submit (
+					E_ALERT_SINK (shell_content),
 					"calendar:failed-open-tasks",
 					error->message, NULL);
 			}
@@ -242,7 +229,7 @@ task_shell_sidebar_default_loaded_cb (ESource *source,
                                       EShellSidebar *shell_sidebar)
 {
 	ETaskShellSidebarPrivate *priv;
-	EShellWindow *shell_window;
+	EShellContent *shell_content;
 	EShellView *shell_view;
 	ECal *client;
 	GError *error = NULL;
@@ -250,7 +237,7 @@ task_shell_sidebar_default_loaded_cb (ESource *source,
 	priv = E_TASK_SHELL_SIDEBAR_GET_PRIVATE (shell_sidebar);
 
 	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
-	shell_window = e_shell_view_get_shell_window (shell_view);
+	shell_content = e_shell_view_get_shell_content (shell_view);
 
 	client = e_load_cal_source_finish (source, result, &error);
 
@@ -259,8 +246,8 @@ task_shell_sidebar_default_loaded_cb (ESource *source,
 		goto exit;
 
 	} else if (error != NULL) {
-		e_alert_run_dialog_for_args (
-			GTK_WINDOW (shell_window),
+		e_alert_submit (
+			E_ALERT_SINK (shell_content),
 			"calendar:failed-open-tasks",
 			error->message, NULL);
 		g_error_free (error);
@@ -348,15 +335,8 @@ task_shell_sidebar_row_changed_cb (ETaskShellSidebar *task_shell_sidebar,
 	ESourceSelector *selector;
 	ESource *source;
 
-	/* XXX ESourceSelector's underlying tree store has only one
-	 *     column: ESource objects.  While we're not supposed to
-	 *     know this, listening for "row-changed" signals from
-	 *     the model is easier to deal with than the selector's
-	 *     "selection-changed" signal, which doesn't tell you
-	 *     _which_ row changed. */
-
 	selector = e_task_shell_sidebar_get_selector (task_shell_sidebar);
-	gtk_tree_model_get (tree_model, tree_iter, 0, &source, -1);
+	source = e_source_selector_get_source_by_path (selector, tree_path);
 
 	/* XXX This signal gets emitted a lot while the model is being
 	 *     rebuilt, during which time we won't get a valid ESource.
@@ -376,12 +356,19 @@ static void
 task_shell_sidebar_selection_changed_cb (ETaskShellSidebar *task_shell_sidebar,
                                          ESourceSelector *selector)
 {
+	EShellView *shell_view;
+	EShellBackend *shell_backend;
+	EShellSidebar *shell_sidebar;
 	GSList *list, *iter;
 
 	/* This signal is emitted less frequently than "row-changed",
 	 * especially when the model is being rebuilt.  So we'll take
 	 * it easy on poor GConf. */
 
+	shell_sidebar = E_SHELL_SIDEBAR (task_shell_sidebar);
+	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
+	shell_backend = e_shell_view_get_shell_backend (shell_view);
+
 	list = e_source_selector_get_selection (selector);
 
 	for (iter = list; iter != NULL; iter = iter->next) {
@@ -391,7 +378,8 @@ task_shell_sidebar_selection_changed_cb (ETaskShellSidebar *task_shell_sidebar,
 		g_object_unref (source);
 	}
 
-	calendar_config_set_tasks_selected (list);
+	e_task_shell_backend_set_selected_task_lists (
+		E_TASK_SHELL_BACKEND (shell_backend), list);
 
 	g_slist_free (list);
 }
@@ -400,31 +388,12 @@ static void
 task_shell_sidebar_primary_selection_changed_cb (ETaskShellSidebar *task_shell_sidebar,
                                                  ESourceSelector *selector)
 {
-	EShell *shell;
-	EShellView *shell_view;
-	EShellWindow *shell_window;
-	EShellSidebar *shell_sidebar;
-	EShellSettings *shell_settings;
 	ESource *source;
 
-	/* XXX ESourceSelector needs a "primary-selection-uid" property
-	 *     so we can just bind the property with GConfBridge. */
-
-	source = e_source_selector_peek_primary_selection (selector);
+	source = e_source_selector_get_primary_selection (selector);
 	if (source == NULL)
 		return;
 
-	shell_sidebar = E_SHELL_SIDEBAR (task_shell_sidebar);
-	shell_view = e_shell_sidebar_get_shell_view (shell_sidebar);
-	shell_window = e_shell_view_get_shell_window (shell_view);
-
-	shell = e_shell_window_get_shell (shell_window);
-	shell_settings = e_shell_get_shell_settings (shell);
-
-	e_shell_settings_set_string (
-		shell_settings, "cal-primary-task-list",
-		e_source_peek_uid (source));
-
 	task_shell_sidebar_set_default (task_shell_sidebar, source);
 }
 
@@ -442,7 +411,6 @@ task_shell_sidebar_restore_state_cb (EShellWindow *shell_window,
 	ESource *source;
 	GtkTreeModel *model;
 	GSList *list, *iter;
-	gchar *uid;
 
 	priv = E_TASK_SHELL_SIDEBAR_GET_PRIVATE (shell_sidebar);
 
@@ -468,28 +436,29 @@ task_shell_sidebar_restore_state_cb (EShellWindow *shell_window,
 		G_CALLBACK (task_shell_sidebar_primary_selection_changed_cb),
 		shell_sidebar);
 
-	source = NULL;
-	uid = e_shell_settings_get_string (
-		shell_settings, "cal-primary-task-list");
-	if (uid != NULL)
-		source = e_source_list_peek_source_by_uid (source_list, uid);
-	if (source == NULL)
-		source = e_source_list_peek_source_any (source_list);
-	if (source != NULL)
-		e_source_selector_set_primary_selection (selector, source);
-	g_free (uid);
+	g_object_bind_property_full (
+		shell_settings, "cal-primary-task-list",
+		selector, "primary-selection",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE,
+		(GBindingTransformFunc) e_binding_transform_uid_to_source,
+		(GBindingTransformFunc) e_binding_transform_source_to_uid,
+		g_object_ref (source_list),
+		(GDestroyNotify) g_object_unref);
+
+	list = e_task_shell_backend_get_selected_task_lists (
+		E_TASK_SHELL_BACKEND (shell_backend));
 
-	list = calendar_config_get_tasks_selected ();
 	for (iter = list; iter != NULL; iter = iter->next) {
-		uid = iter->data;
+		const gchar *uid = iter->data;
+
 		source = e_source_list_peek_source_by_uid (source_list, uid);
-		g_free (uid);
 
-		if (source == NULL)
-			continue;
-
-		e_source_selector_select_source (selector, source);
+		if (source != NULL)
+			e_source_selector_select_source (selector, source);
 	}
+
+	g_slist_foreach (list, (GFunc) g_free, NULL);
 	g_slist_free (list);
 
 	/* Listen for subsequent changes to the selector. */
@@ -635,7 +604,7 @@ task_shell_sidebar_check_state (EShellSidebar *shell_sidebar)
 
 	task_shell_sidebar = E_TASK_SHELL_SIDEBAR (shell_sidebar);
 	selector = e_task_shell_sidebar_get_selector (task_shell_sidebar);
-	source = e_source_selector_peek_primary_selection (selector);
+	source = e_source_selector_get_primary_selection (selector);
 
 	if (source != NULL) {
 		ECal *client;
