@@ -23,6 +23,25 @@
 
 #include "widgets/menus/gal-view-factory-etable.h"
 
+#include "e-util/e-util-private.h"
+
+static void
+task_shell_view_model_row_appended_cb (ETaskShellView *task_shell_view,
+                                       ECalModel *model)
+{
+	ETaskShellSidebar *task_shell_sidebar;
+	ECal *client;
+	ESource *source;
+
+	/* This is the "Click to Add" handler. */
+
+	client = e_cal_model_get_default_client (model);
+	source = e_cal_get_source (client);
+
+	task_shell_sidebar = task_shell_view->priv->task_shell_sidebar;
+	e_task_shell_sidebar_add_source (task_shell_sidebar, source);
+}
+
 static void
 task_shell_view_process_completed_tasks (ETaskShellView *task_shell_view)
 {
@@ -56,27 +75,6 @@ task_shell_view_table_popup_event_cb (EShellView *shell_view,
 	e_shell_view_show_popup_menu (shell_view, widget_path, event);
 }
 
-static void
-task_shell_view_table_user_created_cb (ETaskShellView *task_shell_view,
-                                       ETaskTable *task_table)
-{
-	ETaskShellSidebar *task_shell_sidebar;
-	ECalModel *model;
-	ECal *client;
-	ESource *source;
-
-	/* This is the "Click to Add" handler. */
-
-	model = e_task_table_get_model (task_table);
-	client = e_cal_model_get_default_client (model);
-	source = e_cal_get_source (client);
-
-	task_shell_sidebar = task_shell_view->priv->task_shell_sidebar;
-	e_task_shell_sidebar_add_source (task_shell_sidebar, source);
-
-	e_cal_model_add_client (model, client);
-}
-
 static void
 task_shell_view_selector_client_added_cb (ETaskShellView *task_shell_view,
                                           ECal *client)
@@ -248,104 +246,111 @@ e_task_shell_view_private_constructed (ETaskShellView *task_shell_view)
 	task_shell_sidebar = E_TASK_SHELL_SIDEBAR (shell_sidebar);
 	selector = e_task_shell_sidebar_get_selector (task_shell_sidebar);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		model, "notify::timezone",
 		G_CALLBACK (e_task_shell_view_update_timezone),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
+		model, "row-appended",
+		G_CALLBACK (task_shell_view_model_row_appended_cb),
+		task_shell_view, G_CONNECT_SWAPPED);
+
+	g_signal_connect_object (
 		task_table, "open-component",
 		G_CALLBACK (e_task_shell_view_open_task),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		task_table, "popup-event",
 		G_CALLBACK (task_shell_view_table_popup_event_cb),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		task_table, "selection-change",
 		G_CALLBACK (e_task_shell_view_update_sidebar),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		task_table, "selection-change",
 		G_CALLBACK (e_shell_view_update_actions),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		task_table, "status-message",
 		G_CALLBACK (e_task_shell_view_set_status_message),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
-		task_table, "user-created",
-		G_CALLBACK (task_shell_view_table_user_created_cb),
-		task_shell_view);
-
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		model, "model-changed",
 		G_CALLBACK (e_task_shell_view_update_sidebar),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		model, "model-rows-deleted",
 		G_CALLBACK (e_task_shell_view_update_sidebar),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		model, "model-rows-inserted",
 		G_CALLBACK (e_task_shell_view_update_sidebar),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		task_shell_sidebar, "client-added",
 		G_CALLBACK (task_shell_view_selector_client_added_cb),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		task_shell_sidebar, "client-removed",
 		G_CALLBACK (task_shell_view_selector_client_removed_cb),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		task_shell_sidebar, "status-message",
 		G_CALLBACK (e_task_shell_view_set_status_message),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		selector, "popup-event",
 		G_CALLBACK (task_shell_view_selector_popup_event_cb),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		selector, "primary-selection-changed",
 		G_CALLBACK (e_shell_view_update_actions),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
 	e_categories_add_change_hook (
 		(GHookFunc) e_task_shell_view_update_search_filter,
 		task_shell_view);
 
 	/* Listen for configuration changes. */
-
-	e_mutual_binding_new (
+	g_object_bind_property (
 		shell_settings, "cal-confirm-purge",
-		task_shell_view, "confirm-purge");
+		task_shell_view, "confirm-purge",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE);
+
+	/* Keep the ECalModel in sync with the sidebar. */
+	g_object_bind_property (
+		shell_sidebar, "default-client",
+		model, "default-client",
+		G_BINDING_SYNC_CREATE);
 
 	/* Hide Completed Tasks (enable/units/value) */
-	g_signal_connect_swapped (
+	g_signal_connect_object (
 		shell_settings, "notify::cal-hide-completed-tasks",
 		G_CALLBACK (task_shell_view_process_completed_tasks),
-		task_shell_view);
-	g_signal_connect_swapped (
+		task_shell_view, G_CONNECT_SWAPPED);
+	g_signal_connect_object (
 		shell_settings, "notify::cal-hide-completed-tasks-units",
 		G_CALLBACK (task_shell_view_process_completed_tasks),
-		task_shell_view);
-	g_signal_connect_swapped (
+		task_shell_view, G_CONNECT_SWAPPED);
+	g_signal_connect_object (
 		shell_settings, "notify::cal-hide-completed-tasks-value",
 		G_CALLBACK (task_shell_view_process_completed_tasks),
-		task_shell_view);
+		task_shell_view, G_CONNECT_SWAPPED);
 
 	e_task_shell_view_actions_init (task_shell_view);
 	e_task_shell_view_update_sidebar (task_shell_view);
@@ -372,7 +377,9 @@ e_task_shell_view_private_dispose (ETaskShellView *task_shell_view)
 
 	if (task_shell_view->priv->activity != NULL) {
 		/* XXX Activity is not cancellable. */
-		e_activity_complete (task_shell_view->priv->activity);
+		e_activity_set_state (
+			task_shell_view->priv->activity,
+			E_ACTIVITY_COMPLETED);
 		g_object_unref (task_shell_view->priv->activity);
 		task_shell_view->priv->activity = NULL;
 	}
@@ -516,19 +523,20 @@ e_task_shell_view_set_status_message (ETaskShellView *task_shell_view,
 
 	if (status_message == NULL || *status_message == '\0') {
 		if (activity != NULL) {
-			e_activity_complete (activity);
+			e_activity_set_state (activity, E_ACTIVITY_COMPLETED);
 			g_object_unref (activity);
 			activity = NULL;
 		}
 
 	} else if (activity == NULL) {
-		activity = e_activity_new (status_message);
+		activity = e_activity_new ();
 		e_activity_set_percent (activity, percent);
+		e_activity_set_text (activity, status_message);
 		e_shell_backend_add_activity (shell_backend, activity);
 
 	} else {
 		e_activity_set_percent (activity, percent);
-		e_activity_set_primary_text (activity, status_message);
+		e_activity_set_text (activity, status_message);
 	}
 
 	task_shell_view->priv->activity = activity;
@@ -604,7 +612,5 @@ e_task_shell_view_update_timezone (ETaskShellView *task_shell_view)
 			e_cal_set_default_timezone (client, timezone, NULL);
 	}
 
-	e_cal_component_preview_set_default_timezone (task_preview, timezone);
-
 	g_list_free (clients);
 }
