@@ -46,7 +46,6 @@
 #include "e-mail-backend.h"
 #include "e-mail-label-manager.h"
 #include "e-mail-reader-utils.h"
-#include "mail-config.h"
 #include "em-folder-selection-button.h"
 #include "em-junk.h"
 #include "em-config.h"
@@ -112,6 +111,8 @@ em_mailer_prefs_finalize (GObject *object)
 		prefs->labels_change_notify_id = 0;
 	}
 
+	g_object_unref (prefs->gconf);
+
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (em_mailer_prefs_parent_class)->finalize (object);
 }
@@ -128,7 +129,7 @@ em_mailer_prefs_class_init (EMMailerPrefsClass *class)
 static void
 em_mailer_prefs_init (EMMailerPrefs *preferences)
 {
-	preferences->gconf = mail_config_get_gconf_client ();
+	preferences->gconf = gconf_client_get_default ();
 }
 
 enum {
@@ -297,8 +298,8 @@ emmp_header_remove_sensitivity (EMMailerPrefs *prefs)
 	gboolean is_default;
 
 	/* remove button should be sensitive if the currenlty selected entry in the list view
-           is not a default header. if there are no entries, or none is selected, it should be
-           disabled
+	   is not a default header. if there are no entries, or none is selected, it should be
+	   disabled
 	*/
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
 		gtk_tree_model_get (GTK_TREE_MODEL (prefs->header_list_store), &iter,
@@ -540,20 +541,24 @@ emmp_empty_trash_init (EMMailerPrefs *prefs,
                        GtkComboBox *combo_box)
 {
 	gint days, hist = 0, ii;
-	GtkTreeModel *model;
+	GtkListStore *store;
+	GtkTreeIter iter;
 
 	days = gconf_client_get_int (
 		prefs->gconf,
 		"/apps/evolution/mail/trash/empty_on_exit_days", NULL);
 
-	model = gtk_combo_box_get_model (combo_box);
-	gtk_list_store_clear (GTK_LIST_STORE (model));
+	store = GTK_LIST_STORE (gtk_combo_box_get_model (combo_box));
+	gtk_list_store_clear (store);
 
 	for (ii = 0; ii < G_N_ELEMENTS (empty_trash_frequency); ii++) {
 		if (days >= empty_trash_frequency[ii].days)
 			hist = ii;
-		gtk_combo_box_append_text (
-			combo_box, gettext (empty_trash_frequency[ii].label));
+
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter,
+			0,  gettext (empty_trash_frequency[ii].label),
+			-1);
 	}
 
 	g_signal_connect (
@@ -584,20 +589,24 @@ emmp_empty_junk_init (EMMailerPrefs *prefs,
                       GtkComboBox *combo_box)
 {
 	gint days, hist = 0, ii;
-	GtkTreeModel *model;
+	GtkListStore *store;
+	GtkTreeIter iter;
 
 	days = gconf_client_get_int (
 		prefs->gconf,
 		"/apps/evolution/mail/junk/empty_on_exit_days", NULL);
 
-	model = gtk_combo_box_get_model (combo_box);
-	gtk_list_store_clear (GTK_LIST_STORE (model));
+	store = GTK_LIST_STORE (gtk_combo_box_get_model (combo_box));
+	gtk_list_store_clear (store);
 
 	for (ii = 0; ii < G_N_ELEMENTS (empty_trash_frequency); ii++) {
 		if (days >= empty_trash_frequency[ii].days)
 			hist = ii;
-		gtk_combo_box_append_text (
-			combo_box, gettext (empty_trash_frequency[ii].label));
+
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter,
+			0, gettext (empty_trash_frequency[ii].label),
+			-1);
 	}
 
 	g_signal_connect (
@@ -610,16 +619,16 @@ emmp_empty_junk_init (EMMailerPrefs *prefs,
 static void
 http_images_changed (GtkWidget *widget, EMMailerPrefs *prefs)
 {
-	gint when;
+	EMailImageLoadingPolicy policy;
 
 	if (gtk_toggle_button_get_active (prefs->images_always))
-		when = MAIL_CONFIG_HTTP_ALWAYS;
+		policy = E_MAIL_IMAGE_LOADING_POLICY_ALWAYS;
 	else if (gtk_toggle_button_get_active (prefs->images_sometimes))
-		when = MAIL_CONFIG_HTTP_SOMETIMES;
+		policy = E_MAIL_IMAGE_LOADING_POLICY_SOMETIMES;
 	else
-		when = MAIL_CONFIG_HTTP_NEVER;
+		policy = E_MAIL_IMAGE_LOADING_POLICY_NEVER;
 
-	gconf_client_set_int (prefs->gconf, "/apps/evolution/mail/display/load_http_images", when, NULL);
+	gconf_client_set_int (prefs->gconf, "/apps/evolution/mail/display/load_http_images", policy, NULL);
 }
 
 static GtkWidget *
@@ -659,8 +668,14 @@ emmp_free (EConfig *ec, GSList *items, gpointer data)
 static void
 junk_plugin_changed (GtkWidget *combo, EMMailerPrefs *prefs)
 {
-	gchar *def_plugin = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo));
+	gchar *def_plugin;
 	const GList *plugins = mail_session_get_junk_plugins (prefs->session);
+	GtkTreeIter iter;
+
+	g_return_if_fail (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter));
+
+	def_plugin = NULL;
+	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (combo)), &iter, 0, &def_plugin, -1);
 
 	gconf_client_set_string (prefs->gconf, "/apps/evolution/mail/junk/default_plugin", def_plugin, NULL);
 	while (plugins) {
@@ -693,6 +708,8 @@ junk_plugin_changed (GtkWidget *combo, EMMailerPrefs *prefs)
 		}
 		plugins = plugins->next;
 	}
+
+	g_free (def_plugin);
 }
 
 static void
@@ -817,6 +834,25 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs,
 
 	/* General tab */
 
+	widget = e_builder_get_widget (prefs->builder, "chkCheckMailOnStart");
+	g_object_bind_property (
+		shell_settings, "mail-check-on-start",
+		widget, "active",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE);
+
+	widget = e_builder_get_widget (prefs->builder, "chkCheckMailInAllOnStart");
+	g_object_bind_property (
+		shell_settings, "mail-check-all-on-start",
+		widget, "active",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE);
+	g_object_bind_property (
+		shell_settings, "mail-check-on-start",
+		widget, "sensitive",
+		G_BINDING_BIDIRECTIONAL |
+		G_BINDING_SYNC_CREATE);
+
 	/* Message Display */
 
 	widget = e_builder_get_widget (prefs->builder, "view-check");
@@ -828,6 +864,7 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs,
 
 	widget = e_charset_combo_box_new ();
 	container = e_builder_get_widget (prefs->builder, "hboxDefaultCharset");
+	gtk_label_set_mnemonic_widget (GTK_LABEL (e_builder_get_widget (prefs->builder, "lblDefaultCharset")), widget);
 	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
 	gtk_widget_show (widget);
 	g_object_bind_property (
@@ -928,17 +965,17 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs,
 
 	val = gconf_client_get_int (prefs->gconf, "/apps/evolution/mail/display/load_http_images", NULL);
 	prefs->images_never = GTK_TOGGLE_BUTTON (e_builder_get_widget (prefs->builder, "radImagesNever"));
-	gtk_toggle_button_set_active (prefs->images_never, val == MAIL_CONFIG_HTTP_NEVER);
+	gtk_toggle_button_set_active (prefs->images_never, val == E_MAIL_IMAGE_LOADING_POLICY_NEVER);
 	if (locked)
 		gtk_widget_set_sensitive ((GtkWidget *) prefs->images_never, FALSE);
 
 	prefs->images_sometimes = GTK_TOGGLE_BUTTON (e_builder_get_widget (prefs->builder, "radImagesSometimes"));
-	gtk_toggle_button_set_active (prefs->images_sometimes, val == MAIL_CONFIG_HTTP_SOMETIMES);
+	gtk_toggle_button_set_active (prefs->images_sometimes, val == E_MAIL_IMAGE_LOADING_POLICY_SOMETIMES);
 	if (locked)
 		gtk_widget_set_sensitive ((GtkWidget *) prefs->images_sometimes, FALSE);
 
 	prefs->images_always = GTK_TOGGLE_BUTTON (e_builder_get_widget (prefs->builder, "radImagesAlways"));
-	gtk_toggle_button_set_active (prefs->images_always, val == MAIL_CONFIG_HTTP_ALWAYS);
+	gtk_toggle_button_set_active (prefs->images_always, val == E_MAIL_IMAGE_LOADING_POLICY_ALWAYS);
 	if (locked)
 		gtk_widget_set_sensitive ((GtkWidget *) prefs->images_always, FALSE);
 
@@ -1020,7 +1057,7 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs,
 	renderer = gtk_cell_renderer_toggle_new ();
 	g_object_set (renderer, "activatable", TRUE, NULL);
 	g_signal_connect (renderer, "toggled", G_CALLBACK (emmp_header_list_enabled_toggled), prefs);
-        gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (prefs->header_list), -1,
+	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (prefs->header_list), -1,
 						     "Enabled", renderer,
 						     "active", HEADER_LIST_ENABLED_COLUMN,
 						     NULL);
