@@ -172,14 +172,14 @@ gnome_canvas_item_invoke_point (GnomeCanvasItem *item,
                                 gint cx,
                                 gint cy)
 {
-        cairo_matrix_t inverse;
+	cairo_matrix_t inverse;
 
 	/* Calculate x & y in item local coordinates */
-        inverse = item->matrix;
-        if (cairo_matrix_invert (&inverse) != CAIRO_STATUS_SUCCESS)
-                return NULL;
+	inverse = item->matrix;
+	if (cairo_matrix_invert (&inverse) != CAIRO_STATUS_SUCCESS)
+		return NULL;
 
-        cairo_matrix_transform_point (&inverse, &x, &y);
+	cairo_matrix_transform_point (&inverse, &x, &y);
 
 	if (GNOME_CANVAS_ITEM_GET_CLASS (item)->point)
 		return GNOME_CANVAS_ITEM_GET_CLASS (item)->point (item, x, y, cx, cy);
@@ -272,13 +272,15 @@ pick_current_item (GnomeCanvas *canvas, GdkEvent *event)
 
 		/* world coords */
 
-		x = canvas->scroll_x1 + x / canvas->pixels_per_unit;
-		y = canvas->scroll_y1 + y / canvas->pixels_per_unit;
+		x = canvas->scroll_x1 + x;
+		y = canvas->scroll_y1 + y;
 
 		/* find the closest item */
 
 		if (canvas->root->flags & GNOME_CANVAS_ITEM_VISIBLE)
-			canvas->new_current_item = gnome_canvas_item_invoke_point (canvas->root, x, y, cx, cy);
+			canvas->new_current_item =
+				gnome_canvas_item_invoke_point (
+				canvas->root, x, y, cx, cy);
 		else
 			canvas->new_current_item = NULL;
 	} else
@@ -395,7 +397,7 @@ canvas_realize (GtkWidget *widget)
 	GTK_WIDGET_CLASS (e_canvas_parent_class)->realize (widget);
 
 	window = gtk_layout_get_bin_window (GTK_LAYOUT (widget));
-	gdk_window_set_back_pixmap (window, NULL, FALSE);
+	gdk_window_set_background_pattern (window, NULL);
 
 	window = gtk_widget_get_window (widget);
 	gtk_im_context_set_client_window (ecanvas->im_context, window);
@@ -450,7 +452,7 @@ canvas_button_event (GtkWidget *widget,
 		   canvas->current_item));
 
         /* dispatch normally regardless of the event's window if an item has
-           has a pointer grab in effect */
+	   has a pointer grab in effect */
 	if (!canvas->grabbed_item && event->window != bin_window)
 		return retval;
 
