@@ -34,7 +34,7 @@
 
 #define d(x)
 
-d(static gint depth = 0;)
+d (static gint depth = 0;)
 
 G_DEFINE_TYPE (ETableModel, e_table_model, G_TYPE_OBJECT)
 
@@ -50,7 +50,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint e_table_model_signals [LAST_SIGNAL] = { 0, };
+static guint e_table_model_signals[LAST_SIGNAL] = { 0, };
 
 /**
  * e_table_model_column_count:
@@ -216,7 +216,7 @@ e_table_model_get_save_id (ETableModel *e_table_model, gint row)
 }
 
 gboolean
-e_table_model_has_change_pending(ETableModel *e_table_model)
+e_table_model_has_change_pending (ETableModel *e_table_model)
 {
 	g_return_val_if_fail (e_table_model != NULL, FALSE);
 	g_return_val_if_fail (E_IS_TABLE_MODEL (e_table_model), FALSE);
@@ -271,13 +271,13 @@ e_table_model_finalize (GObject *object)
 }
 
 static void
-e_table_model_class_init (ETableModelClass *klass)
+e_table_model_class_init (ETableModelClass *class)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
 	object_class->finalize = e_table_model_finalize;
 
-	e_table_model_signals [MODEL_NO_CHANGE] =
+	e_table_model_signals[MODEL_NO_CHANGE] =
 		g_signal_new ("model_no_change",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -286,7 +286,7 @@ e_table_model_class_init (ETableModelClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
-	e_table_model_signals [MODEL_CHANGED] =
+	e_table_model_signals[MODEL_CHANGED] =
 		g_signal_new ("model_changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -295,7 +295,7 @@ e_table_model_class_init (ETableModelClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
-	e_table_model_signals [MODEL_PRE_CHANGE] =
+	e_table_model_signals[MODEL_PRE_CHANGE] =
 		g_signal_new ("model_pre_change",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -304,7 +304,7 @@ e_table_model_class_init (ETableModelClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
-	e_table_model_signals [MODEL_ROW_CHANGED] =
+	e_table_model_signals[MODEL_ROW_CHANGED] =
 		g_signal_new ("model_row_changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -313,7 +313,7 @@ e_table_model_class_init (ETableModelClass *klass)
 			      g_cclosure_marshal_VOID__INT,
 			      G_TYPE_NONE, 1, G_TYPE_INT);
 
-	e_table_model_signals [MODEL_CELL_CHANGED] =
+	e_table_model_signals[MODEL_CELL_CHANGED] =
 		g_signal_new ("model_cell_changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -322,7 +322,7 @@ e_table_model_class_init (ETableModelClass *klass)
 			      e_marshal_VOID__INT_INT,
 			      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 
-	e_table_model_signals [MODEL_ROWS_INSERTED] =
+	e_table_model_signals[MODEL_ROWS_INSERTED] =
 		g_signal_new ("model_rows_inserted",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -331,7 +331,7 @@ e_table_model_class_init (ETableModelClass *klass)
 			      e_marshal_VOID__INT_INT,
 			      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 
-	e_table_model_signals [MODEL_ROWS_DELETED] =
+	e_table_model_signals[MODEL_ROWS_DELETED] =
 		g_signal_new ("model_rows_deleted",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -340,31 +340,31 @@ e_table_model_class_init (ETableModelClass *klass)
 			      e_marshal_VOID__INT_INT,
 			      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 
-	klass->column_count        = NULL;
-	klass->row_count           = NULL;
-	klass->append_row          = NULL;
+	class->column_count        = NULL;
+	class->row_count           = NULL;
+	class->append_row          = NULL;
 
-	klass->value_at            = NULL;
-	klass->set_value_at        = NULL;
-	klass->is_cell_editable    = NULL;
+	class->value_at            = NULL;
+	class->set_value_at        = NULL;
+	class->is_cell_editable    = NULL;
 
-	klass->has_save_id         = NULL;
-	klass->get_save_id         = NULL;
+	class->has_save_id         = NULL;
+	class->get_save_id         = NULL;
 
-	klass->has_change_pending  = NULL;
+	class->has_change_pending  = NULL;
 
-	klass->duplicate_value     = NULL;
-	klass->free_value          = NULL;
-	klass->initialize_value    = NULL;
-	klass->value_is_empty      = NULL;
-	klass->value_to_string     = NULL;
+	class->duplicate_value     = NULL;
+	class->free_value          = NULL;
+	class->initialize_value    = NULL;
+	class->value_is_empty      = NULL;
+	class->value_to_string     = NULL;
 
-	klass->model_no_change     = NULL;
-	klass->model_changed       = NULL;
-	klass->model_row_changed   = NULL;
-	klass->model_cell_changed  = NULL;
-	klass->model_rows_inserted = NULL;
-	klass->model_rows_deleted  = NULL;
+	class->model_no_change     = NULL;
+	class->model_changed       = NULL;
+	class->model_row_changed   = NULL;
+	class->model_cell_changed  = NULL;
+	class->model_rows_inserted = NULL;
+	class->model_rows_deleted  = NULL;
 }
 
 static void
@@ -392,12 +392,12 @@ e_table_model_pre_change (ETableModel *e_table_model)
 	if (ETM_FROZEN (e_table_model))
 		return;
 
-	d(print_tabs());
-	d(g_print("Emitting pre_change on model 0x%p, a %s.\n", e_table_model, g_type_name (GTK_OBJECT(e_table_model)->klass->type)));
-	d(depth++);
+	d (print_tabs ());
+	d(g_print("Emitting pre_change on model 0x%p, a %s.\n", e_table_model, g_type_name (G_OBJECT(e_table_model)->class->type)));
+	d (depth++);
 	g_signal_emit (G_OBJECT (e_table_model),
-		       e_table_model_signals [MODEL_PRE_CHANGE], 0);
-	d(depth--);
+		       e_table_model_signals[MODEL_PRE_CHANGE], 0);
+	d (depth--);
 }
 
 /**
@@ -422,12 +422,12 @@ e_table_model_no_change (ETableModel *e_table_model)
 	if (ETM_FROZEN (e_table_model))
 		return;
 
-	d(print_tabs());
-	d(g_print("Emitting model_no_change on model 0x%p, a %s.\n", e_table_model, g_type_name (GTK_OBJECT(e_table_model)->klass->type)));
-	d(depth++);
+	d (print_tabs ());
+	d(g_print("Emitting model_no_change on model 0x%p, a %s.\n", e_table_model, g_type_name (G_OBJECT(e_table_model)->class->type)));
+	d (depth++);
 	g_signal_emit (G_OBJECT (e_table_model),
-		       e_table_model_signals [MODEL_NO_CHANGE], 0);
-	d(depth--);
+		       e_table_model_signals[MODEL_NO_CHANGE], 0);
+	d (depth--);
 }
 
 /**
@@ -452,12 +452,12 @@ e_table_model_changed (ETableModel *e_table_model)
 	if (ETM_FROZEN (e_table_model))
 		return;
 
-	d(print_tabs());
-	d(g_print("Emitting model_changed on model 0x%p, a %s.\n", e_table_model, g_type_name (GTK_OBJECT(e_table_model)->klass->type)));
-	d(depth++);
+	d (print_tabs ());
+	d(g_print("Emitting model_changed on model 0x%p, a %s.\n", e_table_model, g_type_name (G_OBJECT(e_table_model)->class->type)));
+	d (depth++);
 	g_signal_emit (G_OBJECT (e_table_model),
-		       e_table_model_signals [MODEL_CHANGED], 0);
-	d(depth--);
+		       e_table_model_signals[MODEL_CHANGED], 0);
+	d (depth--);
 }
 
 /**
@@ -479,12 +479,12 @@ e_table_model_row_changed (ETableModel *e_table_model, gint row)
 	if (ETM_FROZEN (e_table_model))
 		return;
 
-	d(print_tabs());
-	d(g_print("Emitting row_changed on model 0x%p, a %s, row %d.\n", e_table_model, g_type_name (GTK_OBJECT(e_table_model)->klass->type), row));
-	d(depth++);
+	d (print_tabs ());
+	d(g_print("Emitting row_changed on model 0x%p, a %s, row %d.\n", e_table_model, g_type_name (G_OBJECT(e_table_model)->class->type), row));
+	d (depth++);
 	g_signal_emit (G_OBJECT (e_table_model),
-		       e_table_model_signals [MODEL_ROW_CHANGED], 0, row);
-	d(depth--);
+		       e_table_model_signals[MODEL_ROW_CHANGED], 0, row);
+	d (depth--);
 }
 
 /**
@@ -507,12 +507,12 @@ e_table_model_cell_changed (ETableModel *e_table_model, gint col, gint row)
 	if (ETM_FROZEN (e_table_model))
 		return;
 
-	d(print_tabs());
-	d(g_print("Emitting cell_changed on model 0x%p, a %s, row %d, col %d.\n", e_table_model, g_type_name (GTK_OBJECT(e_table_model)->klass->type), row, col));
-	d(depth++);
+	d (print_tabs ());
+	d(g_print("Emitting cell_changed on model 0x%p, a %s, row %d, col %d.\n", e_table_model, g_type_name (G_OBJECT(e_table_model)->class->type), row, col));
+	d (depth++);
 	g_signal_emit (G_OBJECT (e_table_model),
-		       e_table_model_signals [MODEL_CELL_CHANGED], 0, col, row);
-	d(depth--);
+		       e_table_model_signals[MODEL_CELL_CHANGED], 0, col, row);
+	d (depth--);
 }
 
 /**
@@ -535,12 +535,12 @@ e_table_model_rows_inserted (ETableModel *e_table_model, gint row, gint count)
 	if (ETM_FROZEN (e_table_model))
 		return;
 
-	d(print_tabs());
-	d(g_print("Emitting row_inserted on model 0x%p, a %s, row %d.\n", e_table_model, g_type_name (GTK_OBJECT(e_table_model)->klass->type), row));
-	d(depth++);
+	d (print_tabs ());
+	d(g_print("Emitting row_inserted on model 0x%p, a %s, row %d.\n", e_table_model, g_type_name (G_OBJECT(e_table_model)->class->type), row));
+	d (depth++);
 	g_signal_emit (G_OBJECT (e_table_model),
-		       e_table_model_signals [MODEL_ROWS_INSERTED], 0, row, count);
-	d(depth--);
+		       e_table_model_signals[MODEL_ROWS_INSERTED], 0, row, count);
+	d (depth--);
 }
 
 /**
@@ -555,7 +555,7 @@ e_table_model_rows_inserted (ETableModel *e_table_model, gint row, gint count)
 void
 e_table_model_row_inserted (ETableModel *e_table_model, gint row)
 {
-	e_table_model_rows_inserted(e_table_model, row, 1);
+	e_table_model_rows_inserted (e_table_model, row, 1);
 }
 
 /**
@@ -578,12 +578,12 @@ e_table_model_rows_deleted (ETableModel *e_table_model, gint row, gint count)
 	if (ETM_FROZEN (e_table_model))
 		return;
 
-	d(print_tabs());
-	d(g_print("Emitting row_deleted on model 0x%p, a %s, row %d.\n", e_table_model, g_type_name (GTK_OBJECT(e_table_model)->klass->type), row));
-	d(depth++);
+	d (print_tabs ());
+	d(g_print("Emitting row_deleted on model 0x%p, a %s, row %d.\n", e_table_model, g_type_name (G_OBJECT(e_table_model)->class->type), row));
+	d (depth++);
 	g_signal_emit (G_OBJECT (e_table_model),
-		       e_table_model_signals [MODEL_ROWS_DELETED], 0, row, count);
-	d(depth--);
+		       e_table_model_signals[MODEL_ROWS_DELETED], 0, row, count);
+	d (depth--);
 }
 
 /**
@@ -598,7 +598,7 @@ e_table_model_rows_deleted (ETableModel *e_table_model, gint row, gint count)
 void
 e_table_model_row_deleted (ETableModel *e_table_model, gint row)
 {
-	e_table_model_rows_deleted(e_table_model, row, 1);
+	e_table_model_rows_deleted (e_table_model, row, 1);
 }
 
 void
