@@ -55,9 +55,6 @@
 
 #include "e-text.h"
 
-/* backward-compatibility cruft */
-#include "e-util/gtk-compat.h"
-
 G_DEFINE_TYPE (EText, e_text, GNOME_TYPE_CANVAS_ITEM)
 
 #define BORDER_INDENT 3
@@ -134,6 +131,7 @@ static void e_text_do_popup (EText *text, GdkEventButton *button, gint position)
 static void e_text_update_primary_selection (EText *text);
 static void e_text_paste (EText *text, GdkAtom selection);
 static void e_text_insert (EText *text, const gchar *string);
+static void e_text_reset_im_context (EText *text);
 
 static void reset_layout_attrs (EText *text);
 
@@ -177,6 +175,17 @@ static GdkAtom clipboard_atom = GDK_NONE;
 
 
 
+static void
+disconnect_im_context (EText *text)
+{
+	if (!text || !text->im_context)
+		return;
+
+	g_signal_handlers_disconnect_matched (
+		text->im_context, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, text);
+	text->im_context_signals_registered = FALSE;
+}
+
 /* Dispose handler for the text item */
 
 #if 0
@@ -255,10 +264,7 @@ e_text_dispose (GObject *object)
 	}
 
 	if (text->im_context) {
-		g_signal_handlers_disconnect_matched (text->im_context,
-						      G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL,
-						      NULL, text);
+		disconnect_im_context (text);
 		g_object_unref (text->im_context);
 		text->im_context = NULL;
 	}
@@ -531,8 +537,8 @@ get_bounds (EText *text, gdouble *px1, gdouble *py1, gdouble *px2, gdouble *py2)
 		clip_height = text->clip_height;
 
 	/* Get canvas pixel coordinates for clip rectangle position */
-	text->clip_cwidth = clip_width * item->canvas->pixels_per_unit;
-	text->clip_cheight = clip_height * item->canvas->pixels_per_unit;
+	text->clip_cwidth = clip_width;
+	text->clip_cheight = clip_height;
 
 	text->text_cx = text->cx;
 	text->text_cy = text->cy;
@@ -760,7 +766,7 @@ e_text_set_property (GObject *object,
 		needs_update = 1;
 		break;
 
-        case PROP_FILL_COLOR:
+	case PROP_FILL_COLOR:
 		if (g_value_get_string (value))
 			gdk_color_parse (g_value_get_string (value), &color);
 
@@ -786,7 +792,7 @@ e_text_set_property (GObject *object,
 		needs_update = 1;
 		break;
 
-        case PROP_FILL_COLOR_RGBA:
+	case PROP_FILL_COLOR_RGBA:
 		text->rgba = g_value_get_uint (value);
 		color.red = ((text->rgba >> 24) & 0xff) * 0x101;
 		color.green = ((text->rgba >> 16) & 0xff) * 0x101;
@@ -906,14 +912,16 @@ e_text_set_property (GObject *object,
 	}
 
 	case PROP_IM_CONTEXT:
-		if (text->im_context)
+		if (text->im_context) {
+			disconnect_im_context (text);
 			g_object_unref (text->im_context);
+		}
 
 		text->im_context = g_value_get_object (value);
 		if (text->im_context)
 			g_object_ref (text->im_context);
 
-		text->need_im_reset = FALSE;
+		text->need_im_reset = TRUE;
 		break;
 
 	case PROP_HANDLE_POPUP:
@@ -996,11 +1004,11 @@ e_text_get_property (GObject *object,
 		break;
 
 	case PROP_TEXT_WIDTH:
-		g_value_set_double (value, text->width / text->item.canvas->pixels_per_unit);
+		g_value_set_double (value, text->width);
 		break;
 
 	case PROP_TEXT_HEIGHT:
-		g_value_set_double (value, text->height / text->item.canvas->pixels_per_unit);
+		g_value_set_double (value, text->height);
 		break;
 
 	case PROP_EDITABLE:
@@ -1035,8 +1043,7 @@ e_text_get_property (GObject *object,
 		g_value_set_double (
 			value, text->clip &&
 			text->clip_height != -1 ?
-			text->clip_height : text->height /
-			text->item.canvas->pixels_per_unit);
+			text->clip_height : text->height);
 		break;
 
 	case PROP_DRAW_BORDERS:
@@ -1117,25 +1124,28 @@ e_text_update (GnomeCanvasItem *item, const cairo_matrix_t *i2c, gint flags)
 
 	if ( text->needs_recalc_bounds
 	     || (flags & GNOME_CANVAS_UPDATE_AFFINE)) {
-                get_bounds (text, &x1, &y1, &x2, &y2);
-                if ( item->x1 != x1 ||
-                     item->x2 != x2 ||
-                     item->y1 != y1 ||
-                     item->y2 != y2 ) {
-                        gnome_canvas_request_redraw (item->canvas, item->x1, item->y1, item->x2, item->y2);
-                        item->x1 = x1;
-                        item->y1 = y1;
-                        item->x2 = x2;
-                        item->y2 = y2;
-                        text->needs_redraw = 1;
-                        item->canvas->need_repick = TRUE;
-                }
-                if (!text->fill_clip_rectangle)
-                        item->canvas->need_repick = TRUE;
+		get_bounds (text, &x1, &y1, &x2, &y2);
+		if ( item->x1 != x1 ||
+		     item->x2 != x2 ||
+		     item->y1 != y1 ||
+		     item->y2 != y2 ) {
+			gnome_canvas_request_redraw (
+				item->canvas, item->x1, item->y1,
+				item->x2, item->y2);
+			item->x1 = x1;
+			item->y1 = y1;
+			item->x2 = x2;
+			item->y2 = y2;
+			text->needs_redraw = 1;
+			item->canvas->need_repick = TRUE;
+		}
+		if (!text->fill_clip_rectangle)
+			item->canvas->need_repick = TRUE;
 		text->needs_recalc_bounds = 0;
 	}
 	if (text->needs_redraw) {
-		gnome_canvas_request_redraw (item->canvas, item->x1, item->y1, item->x2, item->y2);
+		gnome_canvas_request_redraw (
+			item->canvas, item->x1, item->y1, item->x2, item->y2);
 		text->needs_redraw = 0;
 	}
 }
@@ -1165,9 +1175,9 @@ e_text_unrealize (GnomeCanvasItem *item)
 
 	text = E_TEXT (item);
 
-	gdk_cursor_unref (text->i_cursor);
+	g_object_unref (text->i_cursor);
 	text->i_cursor = NULL;
-	gdk_cursor_unref (text->default_cursor);
+	g_object_unref (text->default_cursor);
 	text->default_cursor = NULL;
 
 	if (GNOME_CANVAS_ITEM_CLASS (e_text_parent_class)->unrealize)
@@ -1188,20 +1198,23 @@ _get_tep (EText *text)
 }
 
 static void
-draw_pango_rectangle (GdkDrawable *drawable,
-                      cairo_t *cr,
+draw_pango_rectangle (cairo_t *cr,
                       gint x1,
                       gint y1,
                       PangoRectangle rect)
 {
 	gint width = rect.width / PANGO_SCALE;
 	gint height = rect.height / PANGO_SCALE;
+
 	if (width <= 0)
 		width = 1;
 	if (height <= 0)
 		height = 1;
-	cairo_rectangle (cr, x1 + rect.x / PANGO_SCALE, y1 + rect.y / PANGO_SCALE, width, height);
-        cairo_fill (cr);
+
+	cairo_rectangle (
+		cr, x1 + rect.x / PANGO_SCALE,
+		y1 + rect.y / PANGO_SCALE, width, height);
+	cairo_fill (cr);
 }
 
 static gboolean
@@ -1270,12 +1283,11 @@ show_pango_rectangle (EText *text, PangoRectangle rect)
 
 /* Draw handler for the text item */
 static void
-e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
+e_text_draw (GnomeCanvasItem *item, cairo_t *cr,
 	     gint x, gint y, gint width, gint height)
 {
 	EText *text;
 	gint xpos, ypos;
-	cairo_t *cr;
 	GnomeCanvas *canvas;
 	GtkWidget *widget;
 	GdkWindow *window;
@@ -1288,16 +1300,17 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	state = gtk_widget_get_state (widget);
 	style = gtk_widget_get_style (widget);
 	window = gtk_widget_get_window (widget);
-        cr = gdk_cairo_create (drawable);
+
+	cairo_save (cr);
 
 	if (text->draw_background || text->draw_button) {
 		gdk_cairo_set_source_color (cr, &style->fg[state]);
 	} else {
-                cairo_set_source_rgba (cr,
-                                       ((text->rgba >> 24) & 0xff) / 255.0,
-                                       ((text->rgba >> 16) & 0xff) / 255.0,
-                                       ((text->rgba >>  8) & 0xff) / 255.0,
-                                       ( text->rgba        & 0xff) / 255.0);
+		cairo_set_source_rgba (cr,
+				       ((text->rgba >> 24) & 0xff) / 255.0,
+				       ((text->rgba >> 16) & 0xff) / 255.0,
+				       ((text->rgba >>  8) & 0xff) / 255.0,
+				       ( text->rgba        & 0xff) / 255.0);
 	}
 
 	if (text->draw_borders || text->draw_background) {
@@ -1313,17 +1326,17 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 
 		if (text->draw_borders) {
 
-			gtk_paint_shadow (style, drawable,
+			gtk_paint_shadow (style, cr,
 					  GTK_STATE_NORMAL, GTK_SHADOW_IN,
-					  NULL, widget, "entry",
+					  widget, "entry",
 					  thisx, thisy, thiswidth, thisheight);
 
 		}
 
 		if (text->draw_background) {
-			gtk_paint_flat_box (style, drawable,
+			gtk_paint_flat_box (style, cr,
 					    state, GTK_SHADOW_NONE,
-					    NULL, widget, "entry_bg",
+					    widget, "entry_bg",
 					    thisx + style->xthickness,
 					    thisy + style->ythickness,
 					    thiswidth - style->xthickness * 2,
@@ -1357,12 +1370,6 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			guint border_width;
 			gint thisx, thisy, thisheight, thiswidth;
 			gint default_spacing;
-			GdkRectangle area;
-
-			area.x = 0;
-			area.y = 0;
-			area.width = width;
-			area.height = height;
 
 			gtk_widget_get_allocation (widget, &allocation);
 			relief = gtk_button_get_relief (GTK_BUTTON (widget));
@@ -1384,9 +1391,9 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			if (gtk_widget_has_default (widget) &&
 			    relief == GTK_RELIEF_NORMAL)
 				{
-					gtk_paint_box (style, drawable,
+					gtk_paint_box (style, cr,
 						       GTK_STATE_NORMAL, GTK_SHADOW_IN,
-						       &area, widget, "buttondefault",
+						       widget, "buttondefault",
 						       thisx + xoff, thisy + yoff, thiswidth, thisheight);
 				}
 
@@ -1414,8 +1421,8 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			if ((relief != GTK_RELIEF_NONE) ||
 			    ((state != GTK_STATE_NORMAL) &&
 			     (state != GTK_STATE_INSENSITIVE)))
-			gtk_paint_box (style, drawable, state,
-				       shadow_type, &area, widget, "button",
+			gtk_paint_box (style, cr, state,
+				       shadow_type, widget, "button",
 				       thisx + xoff, thisy + yoff,
 				       thiswidth, thisheight);
 
@@ -1425,8 +1432,8 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 				thiswidth += 2;
 				thisheight += 2;
 
-				gtk_paint_focus (style, window, state,
-						 &area, widget, "button",
+				gtk_paint_focus (style, cr, state,
+						 widget, "button",
 						 thisx + xoff, thisy + yoff,
 						 thiswidth - 1, thisheight - 1);
 			}
@@ -1444,7 +1451,7 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 		reset_layout (text);
 
 	if (!pango_layout_get_text (text->layout)) {
-		cairo_destroy (cr);
+		cairo_restore (cr);
 		return;
 	}
 
@@ -1454,12 +1461,14 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	xpos = xpos - x + text->xofs;
 	ypos = ypos - y + text->yofs;
 
+	cairo_save (cr);
+
 	if (text->clip) {
-                cairo_rectangle (cr,
-                                 xpos, ypos,
-                                 text->clip_cwidth - text->xofs,
-                                 text->clip_cheight - text->yofs);
-                cairo_clip (cr);
+		cairo_rectangle (cr,
+				 xpos, ypos,
+				 text->clip_cwidth - text->xofs,
+				 text->clip_cheight - text->yofs);
+		cairo_clip (cr);
 	}
 
 	if (text->editing) {
@@ -1467,16 +1476,19 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 		ypos -= text->yofs_edit;
 	}
 
-        cairo_move_to (cr, xpos, ypos);
-        pango_cairo_show_layout (cr, text->layout);
+	cairo_move_to (cr, xpos, ypos);
+	pango_cairo_show_layout (cr, text->layout);
 
 	if (text->editing) {
 		if (text->selection_start != text->selection_end) {
-			GdkRegion *clip_region = gdk_region_new ();
+			cairo_region_t *clip_region = cairo_region_create ();
 			gint indices[2];
-                        GtkStateType state;
+			GtkStateType state;
 
-                        state = text->has_selection ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
+			if (text->has_selection)
+				state = GTK_STATE_SELECTED;
+			else
+				state = GTK_STATE_ACTIVE;
 
 			indices[0] = MIN (text->selection_start, text->selection_end);
 			indices[1] = MAX (text->selection_start, text->selection_end);
@@ -1485,19 +1497,19 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			indices[0] = g_utf8_offset_to_pointer (text->text, indices[0]) - text->text;
 			indices[1] = g_utf8_offset_to_pointer (text->text, indices[1]) - text->text;
 
-                        clip_region = gdk_pango_layout_get_clip_region (text->layout,
-                                                                        xpos, ypos,
-                                                                        indices, 1);
-                        gdk_cairo_region (cr, clip_region);
-                        cairo_clip (cr);
-			gdk_region_destroy (clip_region);
+			clip_region = gdk_pango_layout_get_clip_region (text->layout,
+									xpos, ypos,
+									indices, 1);
+			gdk_cairo_region (cr, clip_region);
+			cairo_clip (cr);
+			cairo_region_destroy (clip_region);
 
-                        gdk_cairo_set_source_color (cr, &style->base[state]);
-                        cairo_paint (cr);
+			gdk_cairo_set_source_color (cr, &style->base[state]);
+			cairo_paint (cr);
 
-                        gdk_cairo_set_source_color (cr, &style->text[state]);
-                        cairo_move_to (cr, xpos, ypos);
-                        pango_cairo_show_layout (cr, text->layout);
+			gdk_cairo_set_source_color (cr, &style->text[state]);
+			cairo_move_to (cr, xpos, ypos);
+			pango_cairo_show_layout (cr, text->layout);
 		} else {
 			if (text->show_cursor) {
 				PangoRectangle strong_pos, weak_pos;
@@ -1507,17 +1519,18 @@ e_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 					text->layout, offs - text->text +
 					text->preedit_len, &strong_pos,
 					&weak_pos);
-				draw_pango_rectangle (drawable, cr, xpos, ypos, strong_pos);
+				draw_pango_rectangle (cr, xpos, ypos, strong_pos);
 				if (strong_pos.x != weak_pos.x ||
 				    strong_pos.y != weak_pos.y ||
 				    strong_pos.width != weak_pos.width ||
 				    strong_pos.height != weak_pos.height)
-					draw_pango_rectangle (drawable, cr, xpos, ypos, weak_pos);
+					draw_pango_rectangle (cr, xpos, ypos, weak_pos);
 			}
 		}
 	}
 
-	cairo_destroy (cr);
+	cairo_restore (cr);
+	cairo_restore (cr);
 }
 
 /* Point handler for the text item */
@@ -1547,10 +1560,6 @@ e_text_point (GnomeCanvasItem *item, gdouble x, gdouble y,
 	else
 		clip_height = text->clip_height;
 
-	/* Get canvas pixel coordinates for clip rectangle position */
-	clip_width = clip_width * item->canvas->pixels_per_unit;
-	clip_height = clip_height * item->canvas->pixels_per_unit;
-
 	if (cx < text->clip_cx ||
 	    cx > text->clip_cx + clip_width ||
 	    cy < text->clip_cy ||
@@ -1594,9 +1603,6 @@ e_text_bounds (GnomeCanvasItem *item,
 			height = text->clip_height;
 	}
 
-	width = width / item->canvas->pixels_per_unit;
-	height = height / item->canvas->pixels_per_unit;
-
 	*x2 = *x1 + width;
 	*y2 = *y1 + height;
 }
@@ -1720,6 +1726,8 @@ start_editing (EText *text)
 	if (text->editing)
 		return;
 
+	e_text_reset_im_context (text);
+
 	g_free (text->revert);
 	text->revert = g_strdup (text->text);
 
@@ -1768,6 +1776,10 @@ e_text_stop_editing (EText *text)
 		g_timer_destroy (text->timer);
 		text->timer = NULL;
 	}
+
+	text->need_im_reset = TRUE;
+	text->preedit_len = 0;
+	text->preedit_pos = 0;
 }
 
 void
@@ -1791,9 +1803,6 @@ e_text_event (GnomeCanvasItem *item, GdkEvent *event)
 	EText *text = E_TEXT (item);
 	ETextEventProcessorEvent e_tep_event;
 	GdkWindow *window;
-
-	static EText *save_text = NULL;
-
 	gint return_val = 0;
 
 	if (!text->model)
@@ -1808,27 +1817,6 @@ e_text_event (GnomeCanvasItem *item, GdkEvent *event)
 			GdkEventFocus *focus_event;
 			focus_event = (GdkEventFocus *) event;
 			if (focus_event->in) {
-
-				/* Evil hack to disconnect the signal handlers for the previous Etext
-				 * which was not disconnected because of being in preedit mode.
-				 * In preedit mode the widget can go out of focus due to popups associated
-				 * with preedit.,but still the callbacks need to be connected.
-				 * Here when a new text widget comes into focus we can disconnect the
-				 * old one.Shouldn't hurt much, as in worst case, save_text which should
-				 * be disconnected will be overwritten and we will have signal
-				 * handlers connect to  multiple e-texts but with subsequent commit these
-				 * should go away.
-				 */
-
-				if (save_text && save_text->im_context) {
-					g_signal_handlers_disconnect_matched (
-						save_text->im_context,
-						G_SIGNAL_MATCH_DATA,
-                                                0, 0, NULL, NULL, save_text);
-					save_text->im_context_signals_registered =
-						FALSE;
-                                }
-
 				if (text->im_context) {
 					if (!text->im_context_signals_registered) {
 						g_signal_connect (text->im_context, "commit",
@@ -1841,22 +1829,16 @@ e_text_event (GnomeCanvasItem *item, GdkEvent *event)
 								  G_CALLBACK (e_text_delete_surrounding_cb), text);
 						text->im_context_signals_registered = TRUE;
 					}
+					gtk_im_context_focus_in (text->im_context);
 				}
+
 				start_editing (text);
 				text->show_cursor = FALSE; /* so we'll redraw and the cursor will be shown */
 			} else {
-				/* Incase we are not disconnecting the signals
-				 * for text, we are saving text for
-				 * disconnecting in the next focus_in.
-				 */
-				if (!text->preedit_len && text->im_context) {
-					g_signal_handlers_disconnect_matched (text->im_context,
-									      G_SIGNAL_MATCH_DATA,
-									      0, 0, NULL,
-									      NULL, text);
-					text->im_context_signals_registered = FALSE;
-				} else {
-					save_text = text;
+				if (text->im_context) {
+					gtk_im_context_focus_out (text->im_context);
+					disconnect_im_context (text);
+					text->need_im_reset = TRUE;
 				}
 
 				e_text_stop_editing (text);
@@ -1902,8 +1884,7 @@ e_text_event (GnomeCanvasItem *item, GdkEvent *event)
 			gint ret;
 
 			if (text->im_context &&
-				gtk_im_context_filter_keypress (
-				text->im_context, (GdkEventKey*)event)) {
+				gtk_im_context_filter_keypress (text->im_context, (GdkEventKey*) event)) {
 				text->need_im_reset = TRUE;
 				return 1;
 			}
@@ -2097,6 +2078,7 @@ e_text_delete_selection (EText *text)
 
 	if (sel_start != sel_end)
 		e_text_model_delete (text->model, sel_start, sel_end - sel_start);
+	text->need_im_reset = TRUE;
 }
 
 void
@@ -2363,16 +2345,14 @@ e_text_do_popup (EText *text, GdkEventButton *button, gint position)
 					closure);
 }
 
-#if 0
 static void
 e_text_reset_im_context (EText *text)
 {
-	if (text->need_im_reset) {
-		text->need_im_reset = 0;
+	if (text->need_im_reset && text->im_context) {
+		text->need_im_reset = FALSE;
 		gtk_im_context_reset (text->im_context);
 	}
 }
-#endif
 
 /* fixme: */
 
@@ -2880,6 +2860,7 @@ e_text_command (ETextEventProcessor *tep,
 			g_timer_reset (text->timer);
 		}
 
+		text->need_im_reset = TRUE;
 		use_start = TRUE;
 		break;
 	case E_TEP_SELECT:
@@ -2890,6 +2871,7 @@ e_text_command (ETextEventProcessor *tep,
 
 		e_text_update_primary_selection (text);
 
+		text->need_im_reset = TRUE;
 		use_start = FALSE;
 
 		break;
@@ -2902,6 +2884,7 @@ e_text_command (ETextEventProcessor *tep,
 			g_timer_reset (text->timer);
 		}
 
+		text->need_im_reset = TRUE;
 		use_start = FALSE;
 
 		break;
@@ -2915,6 +2898,7 @@ e_text_command (ETextEventProcessor *tep,
 			if (text->timer) {
 				g_timer_reset (text->timer);
 			}
+			text->need_im_reset = TRUE;
 		}
 		break;
 	case E_TEP_COPY:
@@ -2930,6 +2914,7 @@ e_text_command (ETextEventProcessor *tep,
 		if (text->timer) {
 			g_timer_reset (text->timer);
 		}
+		text->need_im_reset = TRUE;
 		break;
 	case E_TEP_GET_SELECTION:
 		e_text_paste (text, GDK_SELECTION_PRIMARY);
@@ -2980,6 +2965,8 @@ e_text_command (ETextEventProcessor *tep,
 		break;
 	}
 
+	e_text_reset_im_context (text);
+
 	/* it's possible to get here without ever having been realized
 	   by our canvas (if the e-text started completely obscured.)
 	   so let's create our layout object if we don't already have
